# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******
from pyVmomi.VmomiSupport import CreateDataType, CreateManagedType, CreateEnumType, AddVersion, AddVersionParent, F_LINK, F_LINKABLE, F_OPTIONAL, F_SECRET
from pyVmomi.VmomiSupport import newestVersions, stableVersions, publicVersions, oldestVersions

AddVersion("vim.version.version10", "vim25", "6.0", 0, "vim25")
AddVersion("vim.version.version11", "vim25", "6.5", 0, "vim25")
AddVersion("vim.version.version12", "vim25", "6.7", 0, "vim25")
AddVersion("vmodl.query.version.version1", "", "", 0, "vim25")
AddVersion("sms.version.version3", "sms", "3.0", 0, "sms")
AddVersion("vmodl.query.version.version2", "", "", 0, "vim25")
AddVersion("sms.version.version4", "sms", "4.0", 0, "sms")
AddVersion("vmodl.query.version.version3", "", "", 0, "vim25")
AddVersion("sms.version.version1", "sms", "1.0", 0, "sms")
AddVersion("vmodl.query.version.version4", "", "", 0, "vim25")
AddVersion("sms.version.version2", "sms", "2.0", 0, "sms")
AddVersion("sms.version.version12", "sms", "6.7", 0, "sms")
AddVersion("sms.version.version5", "sms", "5.0", 0, "sms")
AddVersion("sms.version.version11", "sms", "6.5", 0, "sms")
AddVersion("vmodl.reflect.version.version2", "reflect", "2.0", 0, "reflect")
AddVersion("vmodl.reflect.version.version1", "reflect", "1.0", 0, "reflect")
AddVersion("vim.version.legacy", "vim25", "legacy", 0, "vim25")
AddVersion("vim.version.version1", "vim2", "2.0", 0, "vim25")
AddVersion("vim.version.version2", "vim25", "2.5", 0, "vim25")
AddVersion("vim.version.version3", "vim25", "2.5u2", 0, "vim25")
AddVersion("vim.version.version4", "vim25", "2.5u2server", 0, "vim25")
AddVersion("vim.version.version5", "vim25", "4.0", 0, "vim25")
AddVersion("vim.version.version6", "vim25", "4.1", 0, "vim25")
AddVersion("vim.version.version7", "vim25", "5.0", 0, "vim25")
AddVersion("vim.version.version8", "vim25", "5.1", 0, "vim25")
AddVersion("vim.version.version9", "vim25", "5.5", 0, "vim25")
AddVersion("vmodl.version.version2", "", "", 0, "vim25")
AddVersion("vmodl.version.version1", "", "", 0, "vim25")
AddVersion("sms.version.legacy", "sms", "legacy", 0, "sms")
AddVersion("vmodl.version.version0", "", "", 0, "vim25")
AddVersionParent("vim.version.version10", "vim.version.version10")
AddVersionParent("vim.version.version10", "vmodl.query.version.version1")
AddVersionParent("vim.version.version10", "vmodl.query.version.version2")
AddVersionParent("vim.version.version10", "vmodl.query.version.version3")
AddVersionParent("vim.version.version10", "vmodl.query.version.version4")
AddVersionParent("vim.version.version10", "vmodl.reflect.version.version2")
AddVersionParent("vim.version.version10", "vmodl.reflect.version.version1")
AddVersionParent("vim.version.version10", "vim.version.version1")
AddVersionParent("vim.version.version10", "vim.version.version2")
AddVersionParent("vim.version.version10", "vim.version.version3")
AddVersionParent("vim.version.version10", "vim.version.version4")
AddVersionParent("vim.version.version10", "vim.version.version5")
AddVersionParent("vim.version.version10", "vim.version.version6")
AddVersionParent("vim.version.version10", "vim.version.version7")
AddVersionParent("vim.version.version10", "vim.version.version8")
AddVersionParent("vim.version.version10", "vim.version.version9")
AddVersionParent("vim.version.version10", "vmodl.version.version2")
AddVersionParent("vim.version.version10", "vmodl.version.version1")
AddVersionParent("vim.version.version10", "vmodl.version.version0")
AddVersionParent("vim.version.version11", "vim.version.version10")
AddVersionParent("vim.version.version11", "vim.version.version11")
AddVersionParent("vim.version.version11", "vmodl.query.version.version1")
AddVersionParent("vim.version.version11", "vmodl.query.version.version2")
AddVersionParent("vim.version.version11", "vmodl.query.version.version3")
AddVersionParent("vim.version.version11", "vmodl.query.version.version4")
AddVersionParent("vim.version.version11", "vmodl.reflect.version.version2")
AddVersionParent("vim.version.version11", "vmodl.reflect.version.version1")
AddVersionParent("vim.version.version11", "vim.version.version1")
AddVersionParent("vim.version.version11", "vim.version.version2")
AddVersionParent("vim.version.version11", "vim.version.version3")
AddVersionParent("vim.version.version11", "vim.version.version4")
AddVersionParent("vim.version.version11", "vim.version.version5")
AddVersionParent("vim.version.version11", "vim.version.version6")
AddVersionParent("vim.version.version11", "vim.version.version7")
AddVersionParent("vim.version.version11", "vim.version.version8")
AddVersionParent("vim.version.version11", "vim.version.version9")
AddVersionParent("vim.version.version11", "vmodl.version.version2")
AddVersionParent("vim.version.version11", "vmodl.version.version1")
AddVersionParent("vim.version.version11", "vmodl.version.version0")
AddVersionParent("vim.version.version12", "vim.version.version10")
AddVersionParent("vim.version.version12", "vim.version.version11")
AddVersionParent("vim.version.version12", "vim.version.version12")
AddVersionParent("vim.version.version12", "vmodl.query.version.version1")
AddVersionParent("vim.version.version12", "vmodl.query.version.version2")
AddVersionParent("vim.version.version12", "vmodl.query.version.version3")
AddVersionParent("vim.version.version12", "vmodl.query.version.version4")
AddVersionParent("vim.version.version12", "vmodl.reflect.version.version2")
AddVersionParent("vim.version.version12", "vmodl.reflect.version.version1")
AddVersionParent("vim.version.version12", "vim.version.version1")
AddVersionParent("vim.version.version12", "vim.version.version2")
AddVersionParent("vim.version.version12", "vim.version.version3")
AddVersionParent("vim.version.version12", "vim.version.version4")
AddVersionParent("vim.version.version12", "vim.version.version5")
AddVersionParent("vim.version.version12", "vim.version.version6")
AddVersionParent("vim.version.version12", "vim.version.version7")
AddVersionParent("vim.version.version12", "vim.version.version8")
AddVersionParent("vim.version.version12", "vim.version.version9")
AddVersionParent("vim.version.version12", "vmodl.version.version2")
AddVersionParent("vim.version.version12", "vmodl.version.version1")
AddVersionParent("vim.version.version12", "vmodl.version.version0")
AddVersionParent("vmodl.query.version.version1", "vmodl.query.version.version1")
AddVersionParent("vmodl.query.version.version1", "vmodl.version.version0")
AddVersionParent("sms.version.version3", "vmodl.query.version.version1")
AddVersionParent("sms.version.version3", "sms.version.version3")
AddVersionParent("sms.version.version3", "vmodl.query.version.version2")
AddVersionParent("sms.version.version3", "vmodl.query.version.version3")
AddVersionParent("sms.version.version3", "sms.version.version1")
AddVersionParent("sms.version.version3", "sms.version.version2")
AddVersionParent("sms.version.version3", "vim.version.version1")
AddVersionParent("sms.version.version3", "vim.version.version2")
AddVersionParent("sms.version.version3", "vim.version.version3")
AddVersionParent("sms.version.version3", "vim.version.version4")
AddVersionParent("sms.version.version3", "vim.version.version5")
AddVersionParent("sms.version.version3", "vim.version.version6")
AddVersionParent("sms.version.version3", "vmodl.version.version1")
AddVersionParent("sms.version.version3", "vmodl.version.version0")
AddVersionParent("vmodl.query.version.version2", "vmodl.query.version.version1")
AddVersionParent("vmodl.query.version.version2", "vmodl.query.version.version2")
AddVersionParent("vmodl.query.version.version2", "vmodl.version.version1")
AddVersionParent("vmodl.query.version.version2", "vmodl.version.version0")
AddVersionParent("sms.version.version4", "vmodl.query.version.version1")
AddVersionParent("sms.version.version4", "sms.version.version3")
AddVersionParent("sms.version.version4", "vmodl.query.version.version2")
AddVersionParent("sms.version.version4", "sms.version.version4")
AddVersionParent("sms.version.version4", "vmodl.query.version.version3")
AddVersionParent("sms.version.version4", "sms.version.version1")
AddVersionParent("sms.version.version4", "sms.version.version2")
AddVersionParent("sms.version.version4", "vim.version.version1")
AddVersionParent("sms.version.version4", "vim.version.version2")
AddVersionParent("sms.version.version4", "vim.version.version3")
AddVersionParent("sms.version.version4", "vim.version.version4")
AddVersionParent("sms.version.version4", "vim.version.version5")
AddVersionParent("sms.version.version4", "vim.version.version6")
AddVersionParent("sms.version.version4", "vmodl.version.version1")
AddVersionParent("sms.version.version4", "vmodl.version.version0")
AddVersionParent("vmodl.query.version.version3", "vmodl.query.version.version1")
AddVersionParent("vmodl.query.version.version3", "vmodl.query.version.version2")
AddVersionParent("vmodl.query.version.version3", "vmodl.query.version.version3")
AddVersionParent("vmodl.query.version.version3", "vmodl.version.version1")
AddVersionParent("vmodl.query.version.version3", "vmodl.version.version0")
AddVersionParent("sms.version.version1", "sms.version.version1")
AddVersionParent("sms.version.version1", "vmodl.version.version1")
AddVersionParent("sms.version.version1", "vmodl.version.version0")
AddVersionParent("vmodl.query.version.version4", "vmodl.query.version.version1")
AddVersionParent("vmodl.query.version.version4", "vmodl.query.version.version2")
AddVersionParent("vmodl.query.version.version4", "vmodl.query.version.version3")
AddVersionParent("vmodl.query.version.version4", "vmodl.query.version.version4")
AddVersionParent("vmodl.query.version.version4", "vmodl.version.version2")
AddVersionParent("vmodl.query.version.version4", "vmodl.version.version1")
AddVersionParent("vmodl.query.version.version4", "vmodl.version.version0")
AddVersionParent("sms.version.version2", "vmodl.query.version.version1")
AddVersionParent("sms.version.version2", "vmodl.query.version.version2")
AddVersionParent("sms.version.version2", "vmodl.query.version.version3")
AddVersionParent("sms.version.version2", "sms.version.version1")
AddVersionParent("sms.version.version2", "sms.version.version2")
AddVersionParent("sms.version.version2", "vim.version.version1")
AddVersionParent("sms.version.version2", "vim.version.version2")
AddVersionParent("sms.version.version2", "vim.version.version3")
AddVersionParent("sms.version.version2", "vim.version.version4")
AddVersionParent("sms.version.version2", "vim.version.version5")
AddVersionParent("sms.version.version2", "vim.version.version6")
AddVersionParent("sms.version.version2", "vmodl.version.version1")
AddVersionParent("sms.version.version2", "vmodl.version.version0")
AddVersionParent("sms.version.version12", "vim.version.version10")
AddVersionParent("sms.version.version12", "vim.version.version11")
AddVersionParent("sms.version.version12", "vim.version.version12")
AddVersionParent("sms.version.version12", "vmodl.query.version.version1")
AddVersionParent("sms.version.version12", "sms.version.version3")
AddVersionParent("sms.version.version12", "vmodl.query.version.version2")
AddVersionParent("sms.version.version12", "sms.version.version4")
AddVersionParent("sms.version.version12", "vmodl.query.version.version3")
AddVersionParent("sms.version.version12", "sms.version.version1")
AddVersionParent("sms.version.version12", "vmodl.query.version.version4")
AddVersionParent("sms.version.version12", "sms.version.version2")
AddVersionParent("sms.version.version12", "sms.version.version12")
AddVersionParent("sms.version.version12", "sms.version.version5")
AddVersionParent("sms.version.version12", "sms.version.version11")
AddVersionParent("sms.version.version12", "vmodl.reflect.version.version2")
AddVersionParent("sms.version.version12", "vmodl.reflect.version.version1")
AddVersionParent("sms.version.version12", "vim.version.version1")
AddVersionParent("sms.version.version12", "vim.version.version2")
AddVersionParent("sms.version.version12", "vim.version.version3")
AddVersionParent("sms.version.version12", "vim.version.version4")
AddVersionParent("sms.version.version12", "vim.version.version5")
AddVersionParent("sms.version.version12", "vim.version.version6")
AddVersionParent("sms.version.version12", "vim.version.version7")
AddVersionParent("sms.version.version12", "vim.version.version8")
AddVersionParent("sms.version.version12", "vim.version.version9")
AddVersionParent("sms.version.version12", "vmodl.version.version2")
AddVersionParent("sms.version.version12", "vmodl.version.version1")
AddVersionParent("sms.version.version12", "vmodl.version.version0")
AddVersionParent("sms.version.version5", "vmodl.query.version.version1")
AddVersionParent("sms.version.version5", "sms.version.version3")
AddVersionParent("sms.version.version5", "vmodl.query.version.version2")
AddVersionParent("sms.version.version5", "sms.version.version4")
AddVersionParent("sms.version.version5", "vmodl.query.version.version3")
AddVersionParent("sms.version.version5", "sms.version.version1")
AddVersionParent("sms.version.version5", "sms.version.version2")
AddVersionParent("sms.version.version5", "sms.version.version5")
AddVersionParent("sms.version.version5", "vim.version.version1")
AddVersionParent("sms.version.version5", "vim.version.version2")
AddVersionParent("sms.version.version5", "vim.version.version3")
AddVersionParent("sms.version.version5", "vim.version.version4")
AddVersionParent("sms.version.version5", "vim.version.version5")
AddVersionParent("sms.version.version5", "vim.version.version6")
AddVersionParent("sms.version.version5", "vmodl.version.version1")
AddVersionParent("sms.version.version5", "vmodl.version.version0")
AddVersionParent("sms.version.version11", "vim.version.version10")
AddVersionParent("sms.version.version11", "vim.version.version11")
AddVersionParent("sms.version.version11", "vmodl.query.version.version1")
AddVersionParent("sms.version.version11", "sms.version.version3")
AddVersionParent("sms.version.version11", "vmodl.query.version.version2")
AddVersionParent("sms.version.version11", "sms.version.version4")
AddVersionParent("sms.version.version11", "vmodl.query.version.version3")
AddVersionParent("sms.version.version11", "sms.version.version1")
AddVersionParent("sms.version.version11", "vmodl.query.version.version4")
AddVersionParent("sms.version.version11", "sms.version.version2")
AddVersionParent("sms.version.version11", "sms.version.version5")
AddVersionParent("sms.version.version11", "sms.version.version11")
AddVersionParent("sms.version.version11", "vmodl.reflect.version.version2")
AddVersionParent("sms.version.version11", "vmodl.reflect.version.version1")
AddVersionParent("sms.version.version11", "vim.version.version1")
AddVersionParent("sms.version.version11", "vim.version.version2")
AddVersionParent("sms.version.version11", "vim.version.version3")
AddVersionParent("sms.version.version11", "vim.version.version4")
AddVersionParent("sms.version.version11", "vim.version.version5")
AddVersionParent("sms.version.version11", "vim.version.version6")
AddVersionParent("sms.version.version11", "vim.version.version7")
AddVersionParent("sms.version.version11", "vim.version.version8")
AddVersionParent("sms.version.version11", "vim.version.version9")
AddVersionParent("sms.version.version11", "vmodl.version.version2")
AddVersionParent("sms.version.version11", "vmodl.version.version1")
AddVersionParent("sms.version.version11", "vmodl.version.version0")
AddVersionParent("vmodl.reflect.version.version2", "vmodl.reflect.version.version2")
AddVersionParent("vmodl.reflect.version.version2", "vmodl.reflect.version.version1")
AddVersionParent("vmodl.reflect.version.version2", "vmodl.version.version2")
AddVersionParent("vmodl.reflect.version.version2", "vmodl.version.version1")
AddVersionParent("vmodl.reflect.version.version2", "vmodl.version.version0")
AddVersionParent("vmodl.reflect.version.version1", "vmodl.reflect.version.version1")
AddVersionParent("vmodl.reflect.version.version1", "vmodl.version.version2")
AddVersionParent("vmodl.reflect.version.version1", "vmodl.version.version1")
AddVersionParent("vmodl.reflect.version.version1", "vmodl.version.version0")
AddVersionParent("vim.version.legacy", "vim.version.version10")
AddVersionParent("vim.version.legacy", "vim.version.version11")
AddVersionParent("vim.version.legacy", "vim.version.version12")
AddVersionParent("vim.version.legacy", "vmodl.query.version.version1")
AddVersionParent("vim.version.legacy", "vmodl.query.version.version2")
AddVersionParent("vim.version.legacy", "vmodl.query.version.version3")
AddVersionParent("vim.version.legacy", "vmodl.query.version.version4")
AddVersionParent("vim.version.legacy", "vmodl.reflect.version.version2")
AddVersionParent("vim.version.legacy", "vmodl.reflect.version.version1")
AddVersionParent("vim.version.legacy", "vim.version.legacy")
AddVersionParent("vim.version.legacy", "vim.version.version1")
AddVersionParent("vim.version.legacy", "vim.version.version2")
AddVersionParent("vim.version.legacy", "vim.version.version3")
AddVersionParent("vim.version.legacy", "vim.version.version4")
AddVersionParent("vim.version.legacy", "vim.version.version5")
AddVersionParent("vim.version.legacy", "vim.version.version6")
AddVersionParent("vim.version.legacy", "vim.version.version7")
AddVersionParent("vim.version.legacy", "vim.version.version8")
AddVersionParent("vim.version.legacy", "vim.version.version9")
AddVersionParent("vim.version.legacy", "vmodl.version.version2")
AddVersionParent("vim.version.legacy", "vmodl.version.version1")
AddVersionParent("vim.version.legacy", "vmodl.version.version0")
AddVersionParent("vim.version.version1", "vmodl.query.version.version1")
AddVersionParent("vim.version.version1", "vim.version.version1")
AddVersionParent("vim.version.version1", "vmodl.version.version0")
AddVersionParent("vim.version.version2", "vmodl.query.version.version1")
AddVersionParent("vim.version.version2", "vim.version.version1")
AddVersionParent("vim.version.version2", "vim.version.version2")
AddVersionParent("vim.version.version2", "vmodl.version.version0")
AddVersionParent("vim.version.version3", "vmodl.query.version.version1")
AddVersionParent("vim.version.version3", "vim.version.version1")
AddVersionParent("vim.version.version3", "vim.version.version2")
AddVersionParent("vim.version.version3", "vim.version.version3")
AddVersionParent("vim.version.version3", "vmodl.version.version0")
AddVersionParent("vim.version.version4", "vmodl.query.version.version1")
AddVersionParent("vim.version.version4", "vim.version.version1")
AddVersionParent("vim.version.version4", "vim.version.version2")
AddVersionParent("vim.version.version4", "vim.version.version3")
AddVersionParent("vim.version.version4", "vim.version.version4")
AddVersionParent("vim.version.version4", "vmodl.version.version0")
AddVersionParent("vim.version.version5", "vmodl.query.version.version1")
AddVersionParent("vim.version.version5", "vmodl.query.version.version2")
AddVersionParent("vim.version.version5", "vim.version.version1")
AddVersionParent("vim.version.version5", "vim.version.version2")
AddVersionParent("vim.version.version5", "vim.version.version3")
AddVersionParent("vim.version.version5", "vim.version.version4")
AddVersionParent("vim.version.version5", "vim.version.version5")
AddVersionParent("vim.version.version5", "vmodl.version.version1")
AddVersionParent("vim.version.version5", "vmodl.version.version0")
AddVersionParent("vim.version.version6", "vmodl.query.version.version1")
AddVersionParent("vim.version.version6", "vmodl.query.version.version2")
AddVersionParent("vim.version.version6", "vmodl.query.version.version3")
AddVersionParent("vim.version.version6", "vim.version.version1")
AddVersionParent("vim.version.version6", "vim.version.version2")
AddVersionParent("vim.version.version6", "vim.version.version3")
AddVersionParent("vim.version.version6", "vim.version.version4")
AddVersionParent("vim.version.version6", "vim.version.version5")
AddVersionParent("vim.version.version6", "vim.version.version6")
AddVersionParent("vim.version.version6", "vmodl.version.version1")
AddVersionParent("vim.version.version6", "vmodl.version.version0")
AddVersionParent("vim.version.version7", "vmodl.query.version.version1")
AddVersionParent("vim.version.version7", "vmodl.query.version.version2")
AddVersionParent("vim.version.version7", "vmodl.query.version.version3")
AddVersionParent("vim.version.version7", "vmodl.query.version.version4")
AddVersionParent("vim.version.version7", "vmodl.reflect.version.version1")
AddVersionParent("vim.version.version7", "vim.version.version1")
AddVersionParent("vim.version.version7", "vim.version.version2")
AddVersionParent("vim.version.version7", "vim.version.version3")
AddVersionParent("vim.version.version7", "vim.version.version4")
AddVersionParent("vim.version.version7", "vim.version.version5")
AddVersionParent("vim.version.version7", "vim.version.version6")
AddVersionParent("vim.version.version7", "vim.version.version7")
AddVersionParent("vim.version.version7", "vmodl.version.version2")
AddVersionParent("vim.version.version7", "vmodl.version.version1")
AddVersionParent("vim.version.version7", "vmodl.version.version0")
AddVersionParent("vim.version.version8", "vmodl.query.version.version1")
AddVersionParent("vim.version.version8", "vmodl.query.version.version2")
AddVersionParent("vim.version.version8", "vmodl.query.version.version3")
AddVersionParent("vim.version.version8", "vmodl.query.version.version4")
AddVersionParent("vim.version.version8", "vmodl.reflect.version.version1")
AddVersionParent("vim.version.version8", "vim.version.version1")
AddVersionParent("vim.version.version8", "vim.version.version2")
AddVersionParent("vim.version.version8", "vim.version.version3")
AddVersionParent("vim.version.version8", "vim.version.version4")
AddVersionParent("vim.version.version8", "vim.version.version5")
AddVersionParent("vim.version.version8", "vim.version.version6")
AddVersionParent("vim.version.version8", "vim.version.version7")
AddVersionParent("vim.version.version8", "vim.version.version8")
AddVersionParent("vim.version.version8", "vmodl.version.version2")
AddVersionParent("vim.version.version8", "vmodl.version.version1")
AddVersionParent("vim.version.version8", "vmodl.version.version0")
AddVersionParent("vim.version.version9", "vmodl.query.version.version1")
AddVersionParent("vim.version.version9", "vmodl.query.version.version2")
AddVersionParent("vim.version.version9", "vmodl.query.version.version3")
AddVersionParent("vim.version.version9", "vmodl.query.version.version4")
AddVersionParent("vim.version.version9", "vmodl.reflect.version.version1")
AddVersionParent("vim.version.version9", "vim.version.version1")
AddVersionParent("vim.version.version9", "vim.version.version2")
AddVersionParent("vim.version.version9", "vim.version.version3")
AddVersionParent("vim.version.version9", "vim.version.version4")
AddVersionParent("vim.version.version9", "vim.version.version5")
AddVersionParent("vim.version.version9", "vim.version.version6")
AddVersionParent("vim.version.version9", "vim.version.version7")
AddVersionParent("vim.version.version9", "vim.version.version8")
AddVersionParent("vim.version.version9", "vim.version.version9")
AddVersionParent("vim.version.version9", "vmodl.version.version2")
AddVersionParent("vim.version.version9", "vmodl.version.version1")
AddVersionParent("vim.version.version9", "vmodl.version.version0")
AddVersionParent("vmodl.version.version2", "vmodl.version.version2")
AddVersionParent("vmodl.version.version2", "vmodl.version.version1")
AddVersionParent("vmodl.version.version2", "vmodl.version.version0")
AddVersionParent("vmodl.version.version1", "vmodl.version.version1")
AddVersionParent("vmodl.version.version1", "vmodl.version.version0")
AddVersionParent("sms.version.legacy", "vim.version.version10")
AddVersionParent("sms.version.legacy", "vim.version.version11")
AddVersionParent("sms.version.legacy", "vim.version.version12")
AddVersionParent("sms.version.legacy", "vmodl.query.version.version1")
AddVersionParent("sms.version.legacy", "sms.version.version3")
AddVersionParent("sms.version.legacy", "vmodl.query.version.version2")
AddVersionParent("sms.version.legacy", "sms.version.version4")
AddVersionParent("sms.version.legacy", "vmodl.query.version.version3")
AddVersionParent("sms.version.legacy", "sms.version.version1")
AddVersionParent("sms.version.legacy", "vmodl.query.version.version4")
AddVersionParent("sms.version.legacy", "sms.version.version2")
AddVersionParent("sms.version.legacy", "sms.version.version12")
AddVersionParent("sms.version.legacy", "sms.version.version5")
AddVersionParent("sms.version.legacy", "sms.version.version11")
AddVersionParent("sms.version.legacy", "vmodl.reflect.version.version2")
AddVersionParent("sms.version.legacy", "vmodl.reflect.version.version1")
AddVersionParent("sms.version.legacy", "vim.version.legacy")
AddVersionParent("sms.version.legacy", "vim.version.version1")
AddVersionParent("sms.version.legacy", "vim.version.version2")
AddVersionParent("sms.version.legacy", "vim.version.version3")
AddVersionParent("sms.version.legacy", "vim.version.version4")
AddVersionParent("sms.version.legacy", "vim.version.version5")
AddVersionParent("sms.version.legacy", "vim.version.version6")
AddVersionParent("sms.version.legacy", "vim.version.version7")
AddVersionParent("sms.version.legacy", "vim.version.version8")
AddVersionParent("sms.version.legacy", "vim.version.version9")
AddVersionParent("sms.version.legacy", "vmodl.version.version2")
AddVersionParent("sms.version.legacy", "vmodl.version.version1")
AddVersionParent("sms.version.legacy", "sms.version.legacy")
AddVersionParent("sms.version.legacy", "vmodl.version.version0")
AddVersionParent("vmodl.version.version0", "vmodl.version.version0")

newestVersions.Add("sms.version.version12")
stableVersions.Add("sms.version.version12")
publicVersions.Add("sms.version.version12")
oldestVersions.Add("sms.version.version1")

CreateDataType("sms.AboutInfo", "SmsAboutInfo", "vmodl.DynamicData", "sms.version.version2", [("name", "string", "sms.version.version2", 0), ("fullName", "string", "sms.version.version2", 0), ("vendor", "string", "sms.version.version2", 0), ("apiVersion", "string", "sms.version.version2", 0), ("instanceUuid", "string", "sms.version.version2", 0), ("vasaApiVersion", "string", "sms.version.version4", F_OPTIONAL)])
CreateDataType("sms.EntityReference", "EntityReference", "vmodl.DynamicData", "sms.version.version1", [("id", "string", "sms.version.version1", 0), ("type", "sms.EntityReference.EntityType", "sms.version.version1", F_OPTIONAL)])
CreateEnumType("sms.EntityReference.EntityType", "EntityReferenceEntityType", "sms.version.version1", ["datacenter", "resourcePool", "storagePod", "cluster", "vm", "datastore", "host", "vmFile", "scsiPath", "scsiTarget", "scsiVolume", "scsiAdapter", "nasMount"])
CreateDataType("sms.FaultDomainFilter", "FaultDomainFilter", "vmodl.DynamicData", "sms.version.version11", [("providerId", "string", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.ReplicationGroupFilter", "ReplicationGroupFilter", "vmodl.DynamicData", "sms.version.version11", [("groupId", "vim.vm.replication.ReplicationGroupId[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.ResyncSpec", "ResyncSpec", "vmodl.DynamicData", "sms.version.version4", None)
CreateManagedType("sms.ServiceInstance", "SmsServiceInstance", "vmodl.ManagedObject", "sms.version.version1", None, [("queryStorageManager", "QueryStorageManager", "sms.version.version2", (), (0, "sms.StorageManager", "sms.StorageManager"), "StorageViews.View", None), ("querySessionManager", "QuerySessionManager", "sms.version.version5", (), (0, "sms.auth.SessionManager", "sms.auth.SessionManager"), "System.Anonymous", None), ("queryAboutInfo", "QueryAboutInfo", "sms.version.version2", (), (0, "sms.AboutInfo", "sms.AboutInfo"), "StorageViews.View", None)])
CreateManagedType("sms.StorageManager", "SmsStorageManager", "vmodl.ManagedObject", "sms.version.version2", None, [("registerProvider", "RegisterProvider_Task", "sms.version.version2", (("providerSpec", "sms.provider.ProviderSpec", "sms.version.version2", 0, None),), (0, "sms.Task", "sms.provider.Provider"), "StorageViews.ConfigureService", ["vim.fault.AlreadyExists", "sms.fault.ProviderRegistrationFault", ]), ("unregisterProvider", "UnregisterProvider_Task", "sms.version.version2", (("providerId", "string", "sms.version.version2", 0, None),), (0, "sms.Task", "void"), "StorageViews.ConfigureService", ["vim.fault.NotFound", "sms.fault.ProviderUnregistrationFault", ]), ("queryProvider", "QueryProvider", "sms.version.version2", (), (F_OPTIONAL, "sms.provider.Provider[]", "sms.provider.Provider[]"), "StorageViews.View", ["sms.fault.QueryExecutionFault", ]), ("queryArray", "QueryArray", "sms.version.version2", (("providerId", "string[]", "sms.version.version2", F_OPTIONAL, None),), (F_OPTIONAL, "sms.storage.StorageArray[]", "sms.storage.StorageArray[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryProcessorAssociatedWithArray", "QueryProcessorAssociatedWithArray", "sms.version.version2", (("arrayId", "string", "sms.version.version2", 0, None),), (F_OPTIONAL, "sms.storage.StorageProcessor[]", "sms.storage.StorageProcessor[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryPortAssociatedWithArray", "QueryPortAssociatedWithArray", "sms.version.version2", (("arrayId", "string", "sms.version.version2", 0, None),), (F_OPTIONAL, "sms.storage.StoragePort[]", "sms.storage.StoragePort[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryPortAssociatedWithLun", "QueryPortAssociatedWithLun", "sms.version.version2", (("scsi3Id", "string", "sms.version.version2", 0, None),("arrayId", "string", "sms.version.version2", 0, None),), (F_OPTIONAL, "sms.storage.StoragePort", "sms.storage.StoragePort"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryLunAssociatedWithPort", "QueryLunAssociatedWithPort", "sms.version.version2", (("portId", "string", "sms.version.version2", 0, None),("arrayId", "string", "sms.version.version2", 0, None),), (F_OPTIONAL, "sms.storage.StorageLun[]", "sms.storage.StorageLun[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryArrayAssociatedWithLun", "QueryArrayAssociatedWithLun", "sms.version.version2", (("canonicalName", "string", "sms.version.version2", 0, None),), (F_OPTIONAL, "sms.storage.StorageArray", "sms.storage.StorageArray"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryPortAssociatedWithProcessor", "QueryPortAssociatedWithProcessor", "sms.version.version2", (("processorId", "string", "sms.version.version2", 0, None),("arrayId", "string", "sms.version.version2", 0, None),), (F_OPTIONAL, "sms.storage.StoragePort[]", "sms.storage.StoragePort[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryLunAssociatedWithArray", "QueryLunAssociatedWithArray", "sms.version.version2", (("arrayId", "string", "sms.version.version2", 0, None),), (F_OPTIONAL, "sms.storage.StorageLun[]", "sms.storage.StorageLun[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryFileSystemAssociatedWithArray", "QueryFileSystemAssociatedWithArray", "sms.version.version2", (("arrayId", "string", "sms.version.version2", 0, None),), (F_OPTIONAL, "sms.storage.StorageFileSystem[]", "sms.storage.StorageFileSystem[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryDatastoreCapability", "QueryDatastoreCapability", "sms.version.version2", (("datastore", "vim.Datastore", "sms.version.version2", 0, None),), (F_OPTIONAL, "sms.storage.StorageCapability", "sms.storage.StorageCapability"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryHostAssociatedWithLun", "QueryHostAssociatedWithLun", "sms.version.version2", (("scsi3Id", "string", "sms.version.version2", 0, None),("arrayId", "string", "sms.version.version2", 0, None),), (F_OPTIONAL, "vim.HostSystem[]", "vim.HostSystem[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryVmfsDatastoreAssociatedWithLun", "QueryVmfsDatastoreAssociatedWithLun", "sms.version.version2", (("scsi3Id", "string", "sms.version.version2", 0, None),("arrayId", "string", "sms.version.version2", 0, None),), (F_OPTIONAL, "vim.Datastore", "vim.Datastore"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryNfsDatastoreAssociatedWithFileSystem", "QueryNfsDatastoreAssociatedWithFileSystem", "sms.version.version2", (("fileSystemId", "string", "sms.version.version2", 0, None),("arrayId", "string", "sms.version.version2", 0, None),), (F_OPTIONAL, "vim.Datastore", "vim.Datastore"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryDrsMigrationCapabilityForPerformance", "QueryDrsMigrationCapabilityForPerformance", "sms.version.version2", (("srcDatastore", "vim.Datastore", "sms.version.version2", 0, None),("dstDatastore", "vim.Datastore", "sms.version.version2", 0, None),), (0, "boolean", "boolean"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryDrsMigrationCapabilityForPerformanceEx", "QueryDrsMigrationCapabilityForPerformanceEx", "sms.version.version3", (("datastore", "vim.Datastore[]", "sms.version.version3", 0, None),), (0, "sms.storage.DrsMigrationCapabilityResult", "sms.storage.DrsMigrationCapabilityResult"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryDrsMigrationCapability", "QueryDrsMigrationCapability", "sms.version.version5", (("datastore", "vim.Datastore[]", "sms.version.version5", 0, None),), (0, "sms.storage.DrsMigrationCapabilityResultEx", "sms.storage.DrsMigrationCapabilityResultEx"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryStorageContainer", "QueryStorageContainer", "sms.version.version5", (("containerSpec", "sms.storage.StorageContainerSpec", "sms.version.version5", F_OPTIONAL, None),), (F_OPTIONAL, "sms.storage.StorageContainerResult", "sms.storage.StorageContainerResult"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryProviderUuid", "GetVasaProviderUuid", "sms.version.version4", (("providerId", "string", "sms.version.version4", 0, None),("providerNamespace", "string", "sms.version.version4", 0, None),), (0, "string", "string"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("resyncVsanProviders", "ResyncVsanProviders_Task", "sms.version.version4", (("resyncSpec", "sms.ResyncSpec", "sms.version.version4", F_OPTIONAL, None),), (0, "sms.Task", "void"), "StorageViews.ConfigureService", ["sms.fault.VsanProvidersResyncFailed", ]), ("resyncInternalVasaProvider", "ResyncInternalVasaProvider_Task", "sms.version.version12", (), (0, "sms.Task", "void"), "StorageViews.ConfigureService", ["sms.fault.InternalVasaProviderResyncFailed", ]), ("prepareForBindingChange", "prepareForBindingChange", "sms.version.version5", (("vvolRebindSpec", "sms.storage.VvolRebindSpec", "sms.version.version5", 0, None),("destHostId", "string", "sms.version.version5", 0, None),), (0, "sms.storage.BatchReturnStatus[]", "sms.storage.BatchReturnStatus[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.VvolRebindFault", ]), ("cancelBindingChange", "cancelBindingChange", "sms.version.version5", (("vvolRebindSpec", "sms.storage.VvolRebindSpec", "sms.version.version5", 0, None),), (0, "sms.storage.BatchReturnStatus[]", "sms.storage.BatchReturnStatus[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.VvolRebindFault", ]), ("bindingChangeComplete", "bindingChangeComplete", "sms.version.version5", (("vvolRebindSpec", "sms.storage.VvolRebindSpec", "sms.version.version5", 0, None),), (0, "sms.storage.BatchReturnStatus[]", "sms.storage.BatchReturnStatus[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.VvolRebindFault", ]), ("queryAssociatedBackingStoragePool", "QueryAssociatedBackingStoragePool", "sms.version.version5", (("entityId", "string", "sms.version.version5", F_OPTIONAL, None),("entityType", "string", "sms.version.version5", F_OPTIONAL, None),), (F_OPTIONAL, "sms.storage.BackingStoragePool[]", "sms.storage.BackingStoragePool[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryDatastoreBackingPoolMapping", "QueryDatastoreBackingPoolMapping", "sms.version.version5", (("datastore", "vim.Datastore[]", "sms.version.version5", 0, None),), (0, "sms.storage.DatastoreBackingPoolMapping[]", "sms.storage.DatastoreBackingPoolMapping[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("refreshCACertificatesAndCRLs", "SmsRefreshCACertificatesAndCRLs_Task", "sms.version.version5", (("providerId", "string[]", "sms.version.version5", F_OPTIONAL, None),), (0, "sms.Task", "void"), "StorageViews.ConfigureService", ["vim.fault.NotFound", "sms.fault.CertificateRefreshFailed", ]), ("queryFaultDomain", "QueryFaultDomain", "sms.version.version11", (("filter", "sms.FaultDomainFilter", "sms.version.version11", F_OPTIONAL, None),), (F_OPTIONAL, "vim.vm.replication.FaultDomainId[]", "vim.vm.replication.FaultDomainId[]"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.QueryExecutionFault", ]), ("queryReplicationGroupInfo", "QueryReplicationGroupInfo", "sms.version.version11", (("rgFilter", "sms.ReplicationGroupFilter", "sms.version.version11", 0, None),), (F_OPTIONAL, "sms.storage.replication.GroupOperationResult[]", "sms.storage.replication.GroupOperationResult[]"), "StorageViews.View", ["sms.fault.QueryExecutionFault", ])])
CreateManagedType("sms.Task", "SmsTask", "vmodl.ManagedObject", "sms.version.version2", None, [("queryResult", "QuerySmsTaskResult", "sms.version.version2", (), (F_OPTIONAL, "anyType", "anyType"), "StorageViews.View", None), ("queryInfo", "QuerySmsTaskInfo", "sms.version.version2", (), (0, "sms.TaskInfo", "sms.TaskInfo"), "StorageViews.View", None)])
CreateDataType("sms.TaskInfo", "SmsTaskInfo", "vmodl.DynamicData", "sms.version.version2", [("key", "string", "sms.version.version2", 0), ("task", "sms.Task", "sms.version.version2", 0), ("object", "vmodl.ManagedObject", "sms.version.version2", F_OPTIONAL), ("error", "vmodl.MethodFault", "sms.version.version2", F_OPTIONAL), ("result", "anyType", "sms.version.version2", F_OPTIONAL), ("startTime", "vmodl.DateTime", "sms.version.version2", F_OPTIONAL), ("completionTime", "vmodl.DateTime", "sms.version.version2", F_OPTIONAL), ("state", "string", "sms.version.version2", 0), ("progress", "int", "sms.version.version2", F_OPTIONAL)])
CreateEnumType("sms.TaskInfo.State", "SmsTaskState", "sms.version.version1", ["queued", "running", "success", "error"])
CreateManagedType("sms.auth.SessionManager", "SmsSessionManager", "vmodl.ManagedObject", "sms.version.version5", None, [("loginByToken", "SmsLoginByToken", "sms.version.version5", (("delegatedTokenXml", "string", "sms.version.version5", 0, None),), (0, "void", "void"), "System.Anonymous", ["sms.fault.InvalidLogin", ]), ("logout", "SmsLogout", "sms.version.version5", (), (0, "void", "void"), "StorageViews.View", None)])
CreateDataType("sms.fault.AuthConnectionFailed", "AuthConnectionFailed", "vim.fault.NoPermission", "sms.version.version2", None)
CreateDataType("sms.fault.CertificateRefreshFailed", "CertificateRefreshFailed", "vmodl.MethodFault", "sms.version.version5", [("providerId", "string[]", "sms.version.version5", F_OPTIONAL)])
CreateDataType("sms.fault.CertificateRevocationFailed", "CertificateRevocationFailed", "vmodl.MethodFault", "sms.version.version5", None)
CreateDataType("sms.fault.DuplicateEntry", "DuplicateEntry", "vmodl.MethodFault", "sms.version.version11", None)
CreateDataType("sms.fault.InactiveProvider", "InactiveProvider", "vmodl.MethodFault", "sms.version.version11", [("mapping", "sms.storage.FaultDomainProviderMapping[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.fault.InvalidLogin", "SmsInvalidLogin", "vmodl.MethodFault", "sms.version.version5", None)
CreateDataType("sms.fault.InvalidProfile", "InvalidProfile", "vmodl.MethodFault", "sms.version.version11", None)
CreateDataType("sms.fault.InvalidSession", "InvalidSession", "vim.fault.NoPermission", "sms.version.version2", [("sessionCookie", "string", "sms.version.version2", 0)])
CreateDataType("sms.fault.MultipleSortSpecsNotSupported", "MultipleSortSpecsNotSupported", "vmodl.fault.InvalidArgument", "sms.version.version1", None)
CreateDataType("sms.fault.NotSupportedByProvider", "NotSupportedByProvider", "vmodl.MethodFault", "sms.version.version11", None)
CreateDataType("sms.fault.ProviderBusy", "ProviderBusy", "vmodl.MethodFault", "sms.version.version11", None)
CreateDataType("sms.fault.ProviderConnectionFailed", "ProviderConnectionFailed", "vmodl.RuntimeFault", "sms.version.version2", None)
CreateDataType("sms.fault.ProviderOutOfProvisioningResource", "ProviderOutOfProvisioningResource", "vmodl.MethodFault", "sms.version.version11", [("provisioningResourceId", "string", "sms.version.version11", 0), ("availableBefore", "long", "sms.version.version11", F_OPTIONAL), ("availableAfter", "long", "sms.version.version11", F_OPTIONAL), ("total", "long", "sms.version.version11", F_OPTIONAL), ("isTransient", "boolean", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.fault.ProviderOutOfResource", "ProviderOutOfResource", "vmodl.MethodFault", "sms.version.version11", None)
CreateDataType("sms.fault.ProviderRegistrationFault", "ProviderRegistrationFault", "vmodl.MethodFault", "sms.version.version2", None)
CreateDataType("sms.fault.ProviderSyncFailed", "ProviderSyncFailed", "vmodl.MethodFault", "sms.version.version1", None)
CreateDataType("sms.fault.ProviderUnavailable", "ProviderUnavailable", "vmodl.MethodFault", "sms.version.version11", None)
CreateDataType("sms.fault.ProviderUnregistrationFault", "ProviderUnregistrationFault", "vmodl.MethodFault", "sms.version.version2", None)
CreateDataType("sms.fault.ProxyRegistrationFailed", "ProxyRegistrationFailed", "vmodl.RuntimeFault", "sms.version.version1", None)
CreateDataType("sms.fault.QueryExecutionFault", "QueryExecutionFault", "vmodl.MethodFault", "sms.version.version1", None)
CreateDataType("sms.fault.QueryNotSupported", "QueryNotSupported", "vmodl.fault.InvalidArgument", "sms.version.version1", [("entityType", "sms.EntityReference.EntityType", "sms.version.version1", F_OPTIONAL), ("relatedEntityType", "sms.EntityReference.EntityType", "sms.version.version1", 0)])
CreateDataType("sms.fault.ResourceInUse", "SmsResourceInUse", "vim.fault.ResourceInUse", "sms.version.version11", [("deviceIds", "sms.storage.replication.DeviceId[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.fault.ServiceNotInitialized", "ServiceNotInitialized", "vmodl.RuntimeFault", "sms.version.version1", None)
CreateDataType("sms.fault.SmsFault", "SmsFault", "vmodl.MethodFault", "sms.version.version12", None)
CreateDataType("sms.fault.SyncInProgress", "SyncInProgress", "sms.fault.ProviderSyncFailed", "sms.version.version1", None)
CreateDataType("sms.fault.TooMany", "TooMany", "vmodl.MethodFault", "sms.version.version11", [("maxBatchSize", "long", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.fault.VsanProvidersResyncFailed", "VsanProvidersResyncFailed", "vmodl.MethodFault", "sms.version.version4", [("providerUrl", "string[]", "sms.version.version4", 0)])
CreateDataType("sms.fault.VvolRebindFault", "VvolRebindFault", "vmodl.MethodFault", "sms.version.version5", None)
CreateDataType("sms.fault.replication.ReplicationFault", "SmsReplicationFault", "vmodl.MethodFault", "sms.version.version11", None)
CreateDataType("sms.fault.replication.SyncOngoing", "SyncOngoing", "sms.fault.replication.ReplicationFault", "sms.version.version11", [("task", "sms.Task", "sms.version.version11", 0)])
CreateDataType("sms.provider.AlarmFilter", "AlarmFilter", "vmodl.DynamicData", "sms.version.version11", [("alarmStatus", "string", "sms.version.version11", F_OPTIONAL), ("alarmType", "string", "sms.version.version11", F_OPTIONAL), ("entityType", "string", "sms.version.version11", F_OPTIONAL), ("entityId", "anyType[]", "sms.version.version11", F_OPTIONAL), ("pageMarker", "string", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.provider.AlarmResult", "AlarmResult", "vmodl.DynamicData", "sms.version.version11", [("storageAlarm", "sms.storage.StorageAlarm[]", "sms.version.version11", F_OPTIONAL), ("pageMarker", "string", "sms.version.version11", F_OPTIONAL)])
CreateManagedType("sms.provider.Provider", "SmsProvider", "vmodl.ManagedObject", "sms.version.version2", None, [("queryProviderInfo", "QueryProviderInfo", "sms.version.version2", (), (0, "sms.provider.ProviderInfo", "sms.provider.ProviderInfo"), "StorageViews.View", None)])
CreateDataType("sms.provider.ProviderInfo", "SmsProviderInfo", "vmodl.DynamicData", "sms.version.version2", [("uid", "string", "sms.version.version2", 0), ("name", "string", "sms.version.version2", 0), ("description", "string", "sms.version.version2", F_OPTIONAL), ("version", "string", "sms.version.version2", F_OPTIONAL)])
CreateDataType("sms.provider.ProviderSpec", "SmsProviderSpec", "vmodl.DynamicData", "sms.version.version2", [("name", "string", "sms.version.version2", 0), ("description", "string", "sms.version.version2", F_OPTIONAL)])
CreateManagedType("sms.provider.VasaProvider", "VasaProvider", "sms.provider.Provider", "sms.version.version2", None, [("sync", "VasaProviderSync_Task", "sms.version.version2", (("arrayId", "string", "sms.version.version2", F_OPTIONAL, None),), (0, "sms.Task", "void"), "StorageViews.View", ["sms.fault.ProviderSyncFailed", ]), ("refreshCertificate", "VasaProviderRefreshCertificate_Task", "sms.version.version5", (), (0, "sms.Task", "void"), "StorageViews.ConfigureService", ["sms.fault.CertificateRefreshFailed", ]), ("revokeCertificate", "VasaProviderRevokeCertificate_Task", "sms.version.version5", (), (0, "sms.Task", "void"), "StorageViews.ConfigureService", ["sms.fault.CertificateRevocationFailed", ]), ("reconnect", "VasaProviderReconnect_Task", "sms.version.version5", (), (0, "sms.Task", "void"), "StorageViews.ConfigureService", ["sms.fault.InvalidCertificate", "sms.fault.ProviderConnectionFailed", ]), ("disconnect", "VasaProviderDisconnect_Task", "sms.version.version5", (), (0, "sms.Task", "void"), "StorageViews.ConfigureService", None), ("queryReplicationPeer", "QueryReplicationPeer", "sms.version.version11", (("faultDomainId", "vim.vm.replication.FaultDomainId[]", "sms.version.version11", F_OPTIONAL, None),), (F_OPTIONAL, "sms.storage.replication.QueryReplicationPeerResult[]", "sms.storage.replication.QueryReplicationPeerResult[]"), "StorageViews.View", ["sms.fault.ProviderUnavailable", "sms.fault.InactiveProvider", "sms.fault.ProviderBusy", "sms.fault.QueryExecutionFault", ]), ("queryReplicationGroup", "QueryReplicationGroup", "sms.version.version11", (("groupId", "vim.vm.replication.ReplicationGroupId[]", "sms.version.version11", F_OPTIONAL, None),), (F_OPTIONAL, "sms.storage.replication.GroupOperationResult[]", "sms.storage.replication.GroupOperationResult[]"), "StorageViews.View", ["sms.fault.ProviderUnavailable", "sms.fault.InactiveProvider", "sms.fault.ProviderBusy", "sms.fault.QueryExecutionFault", ]), ("queryPointInTimeReplica", "QueryPointInTimeReplica", "sms.version.version11", (("groupId", "vim.vm.replication.ReplicationGroupId[]", "sms.version.version11", F_OPTIONAL, None),("queryParam", "sms.storage.replication.QueryPointInTimeReplicaParam", "sms.version.version11", F_OPTIONAL, None),), (F_OPTIONAL, "sms.storage.replication.GroupOperationResult[]", "sms.storage.replication.GroupOperationResult[]"), "StorageViews.View", ["sms.fault.ProviderUnavailable", "sms.fault.InactiveProvider", "sms.fault.ProviderBusy", "sms.fault.QueryExecutionFault", ]), ("testFailoverReplicationGroupStart", "TestFailoverReplicationGroupStart_Task", "sms.version.version11", (("testFailoverParam", "sms.storage.replication.TestFailoverParam", "sms.version.version11", 0, None),), (0, "sms.Task", "void"), "StorageViews.ConfigureService", ["sms.fault.ProviderUnavailable", "sms.fault.ProviderOutOfResource", "sms.fault.InactiveProvider", "sms.fault.TooMany", "sms.fault.ProviderBusy", "sms.fault.replication.ReplicationFault", ]), ("testFailoverReplicationGroupStop", "TestFailoverReplicationGroupStop_Task", "sms.version.version11", (("groupId", "vim.vm.replication.ReplicationGroupId[]", "sms.version.version11", F_OPTIONAL, None),("force", "boolean", "sms.version.version11", 0, None),), (0, "sms.Task", "void"), "StorageViews.ConfigureService", ["sms.fault.ProviderUnavailable", "sms.fault.ProviderOutOfResource", "sms.fault.InactiveProvider", "sms.fault.TooMany", "sms.fault.ProviderBusy", "sms.fault.replication.ReplicationFault", "sms.fault.NotSupportedByProvider", ]), ("promoteReplicationGroup", "PromoteReplicationGroup_Task", "sms.version.version11", (("promoteParam", "sms.storage.replication.PromoteParam", "sms.version.version11", 0, None),), (F_OPTIONAL, "sms.Task", "sms.storage.replication.GroupOperationResult[]"), "StorageViews.ConfigureService", ["sms.fault.ProviderUnavailable", "sms.fault.ProviderOutOfResource", "sms.fault.InactiveProvider", "sms.fault.TooMany", "sms.fault.ProviderBusy", "sms.fault.replication.ReplicationFault", ]), ("syncReplicationGroup", "SyncReplicationGroup_Task", "sms.version.version11", (("groupId", "vim.vm.replication.ReplicationGroupId[]", "sms.version.version11", F_OPTIONAL, None),("pitName", "string", "sms.version.version11", 0, None),), (F_OPTIONAL, "sms.Task", "sms.storage.replication.GroupOperationResult[]"), "StorageViews.ConfigureService", ["sms.fault.ProviderUnavailable", "sms.fault.ProviderOutOfResource", "sms.fault.InactiveProvider", "sms.fault.ProviderBusy", "sms.fault.replication.ReplicationFault", "sms.fault.TooMany", ]), ("prepareFailoverReplicationGroup", "PrepareFailoverReplicationGroup_Task", "sms.version.version11", (("groupId", "vim.vm.replication.ReplicationGroupId[]", "sms.version.version11", F_OPTIONAL, None),), (F_OPTIONAL, "sms.Task", "sms.storage.replication.GroupOperationResult[]"), "StorageViews.ConfigureService", ["sms.fault.ProviderUnavailable", "sms.fault.ProviderOutOfResource", "sms.fault.InactiveProvider", "sms.fault.TooMany", "sms.fault.ProviderBusy", "sms.fault.replication.ReplicationFault", ]), ("failoverReplicationGroup", "FailoverReplicationGroup_Task", "sms.version.version11", (("failoverParam", "sms.storage.replication.FailoverParam", "sms.version.version11", 0, None),), (F_OPTIONAL, "sms.Task", "sms.storage.replication.GroupOperationResult[]"), "StorageViews.ConfigureService", ["sms.fault.ProviderUnavailable", "sms.fault.ProviderOutOfResource", "sms.fault.InactiveProvider", "sms.fault.TooMany", "sms.fault.ProviderBusy", "sms.fault.replication.ReplicationFault", ]), ("reverseReplicateGroup", "ReverseReplicateGroup_Task", "sms.version.version11", (("groupId", "vim.vm.replication.ReplicationGroupId[]", "sms.version.version11", F_OPTIONAL, None),), (F_OPTIONAL, "sms.Task", "sms.storage.replication.GroupOperationResult[]"), "StorageViews.ConfigureService", ["sms.fault.ProviderUnavailable", "sms.fault.ProviderOutOfResource", "sms.fault.InactiveProvider", "sms.fault.TooMany", "sms.fault.ProviderBusy", "sms.fault.replication.ReplicationFault", ]), ("queryActiveAlarm", "QueryActiveAlarm", "sms.version.version11", (("alarmFilter", "sms.provider.AlarmFilter", "sms.version.version11", F_OPTIONAL, None),), (F_OPTIONAL, "sms.provider.AlarmResult", "sms.provider.AlarmResult"), "StorageViews.View", ["vim.fault.NotFound", "sms.fault.ProviderBusy", "sms.fault.InactiveProvider", "sms.fault.ProviderUnavailable", "sms.fault.QueryExecutionFault", ])])
CreateDataType("sms.provider.VasaProviderInfo", "VasaProviderInfo", "sms.provider.ProviderInfo", "sms.version.version2", [("url", "string", "sms.version.version2", 0), ("certificate", "string", "sms.version.version2", F_OPTIONAL), ("status", "string", "sms.version.version2", F_OPTIONAL), ("statusFault", "vmodl.MethodFault", "sms.version.version11", F_OPTIONAL), ("vasaVersion", "string", "sms.version.version2", F_OPTIONAL), ("namespace", "string", "sms.version.version2", F_OPTIONAL), ("lastSyncTime", "string", "sms.version.version2", F_OPTIONAL), ("supportedVendorModelMapping", "sms.provider.VasaProviderInfo.SupportedVendorModelMapping[]", "sms.version.version2", F_OPTIONAL), ("supportedProfile", "string[]", "sms.version.version2", F_OPTIONAL), ("supportedProviderProfile", "string[]", "sms.version.version4", F_OPTIONAL), ("relatedStorageArray", "sms.provider.VasaProviderInfo.RelatedStorageArray[]", "sms.version.version4", F_OPTIONAL), ("providerId", "string", "sms.version.version4", F_OPTIONAL), ("certificateExpiryDate", "string", "sms.version.version5", F_OPTIONAL), ("certificateStatus", "string", "sms.version.version5", F_OPTIONAL), ("serviceLocation", "string", "sms.version.version5", F_OPTIONAL), ("needsExplicitActivation", "boolean", "sms.version.version5", F_OPTIONAL), ("maxBatchSize", "long", "sms.version.version11", F_OPTIONAL), ("retainVasaProviderCertificate", "boolean", "sms.version.version5", F_OPTIONAL), ("arrayIndependentProvider", "boolean", "sms.version.version11", F_OPTIONAL), ("type", "string", "sms.version.version11", F_OPTIONAL), ("category", "string", "sms.version.version12", F_OPTIONAL), ("priority", "int", "sms.version.version11", F_OPTIONAL), ("failoverGroupId", "string", "sms.version.version11", F_OPTIONAL)])
CreateEnumType("sms.provider.VasaProviderInfo.CertificateStatus", "VasaProviderCertificateStatus", "sms.version.version5", ["valid", "expirySoftLimitReached", "expiryHardLimitReached", "expired", "invalid"])
CreateDataType("sms.provider.VasaProviderInfo.RelatedStorageArray", "RelatedStorageArray", "vmodl.DynamicData", "sms.version.version4", [("arrayId", "string", "sms.version.version4", 0), ("active", "boolean", "sms.version.version4", 0), ("manageable", "boolean", "sms.version.version4", 0), ("priority", "int", "sms.version.version4", 0)])
CreateDataType("sms.provider.VasaProviderInfo.SupportedVendorModelMapping", "SupportedVendorModelMapping", "vmodl.DynamicData", "sms.version.version1", [("vendorId", "string", "sms.version.version1", F_OPTIONAL), ("modelId", "string", "sms.version.version1", F_OPTIONAL)])
CreateEnumType("sms.provider.VasaProviderInfo.VasaProviderStatus", "VasaProviderStatus", "sms.version.version1", ["online", "offline", "syncError", "unknown", "connected", "disconnected"])
CreateEnumType("sms.provider.VasaProviderInfo.VasaProviderProfile", "VasaProviderProfile", "sms.version.version1", ["blockDevice", "fileSystem", "capability"])
CreateEnumType("sms.provider.VasaProviderInfo.ProviderProfile", "ProviderProfile", "sms.version.version11", ["ProfileBasedManagement", "Replication"])
CreateEnumType("sms.provider.VasaProviderInfo.Type", "VpType", "sms.version.version11", ["PERSISTENCE", "DATASERVICE", "UNKNOWN"])
CreateEnumType("sms.provider.VasaProviderInfo.Category", "VpCategory", "sms.version.version12", ["internal", "external"])
CreateDataType("sms.provider.VasaProviderSpec", "VasaProviderSpec", "sms.provider.ProviderSpec", "sms.version.version2", [("username", "string", "sms.version.version2", 0), ("password", "string", "sms.version.version2", F_SECRET), ("url", "string", "sms.version.version2", 0), ("certificate", "string", "sms.version.version2", F_OPTIONAL)])
CreateEnumType("sms.storage.AlarmStatus", "SmsAlarmStatus", "sms.version.version11", ["Red", "Green", "Yellow"])
CreateEnumType("sms.storage.AlarmType", "AlarmType", "sms.version.version11", ["SpaceCapacityAlarm", "CapabilityAlarm", "StorageObjectAlarm", "ObjectAlarm", "ComplianceAlarm", "ManageabilityAlarm", "ReplicationAlarm"])
CreateDataType("sms.storage.BackingConfig", "BackingConfig", "vmodl.DynamicData", "sms.version.version5", [("thinProvisionBackingIdentifier", "string", "sms.version.version5", F_OPTIONAL), ("deduplicationBackingIdentifier", "string", "sms.version.version5", F_OPTIONAL), ("autoTieringEnabled", "boolean", "sms.version.version5", F_OPTIONAL), ("deduplicationEfficiency", "long", "sms.version.version5", F_OPTIONAL), ("performanceOptimizationInterval", "long", "sms.version.version5", F_OPTIONAL)])
CreateDataType("sms.storage.BackingStoragePool", "BackingStoragePool", "vmodl.DynamicData", "sms.version.version5", [("uuid", "string", "sms.version.version5", 0), ("type", "string", "sms.version.version5", 0), ("capacityInMB", "long", "sms.version.version5", 0), ("usedSpaceInMB", "long", "sms.version.version5", 0)])
CreateEnumType("sms.storage.BackingStoragePool.BackingStoragePoolType", "BackingStoragePoolType", "sms.version.version1", ["thinProvisioningPool", "deduplicationPool", "thinAndDeduplicationCombinedPool"])
CreateDataType("sms.storage.BatchErrorResult", "BatchErrorResult", "vmodl.DynamicData", "sms.version.version5", [("error", "vmodl.MethodFault[]", "sms.version.version5", 0)])
CreateDataType("sms.storage.BatchReturnStatus", "BatchReturnStatus", "vmodl.DynamicData", "sms.version.version5", [("uniqueId", "string", "sms.version.version5", 0), ("returnStatus", "sms.storage.BatchErrorResult", "sms.version.version5", F_OPTIONAL)])
CreateDataType("sms.storage.DatastoreBackingPoolMapping", "DatastoreBackingPoolMapping", "vmodl.DynamicData", "sms.version.version5", [("datastore", "vim.Datastore[]", "sms.version.version5", 0), ("backingStoragePool", "sms.storage.BackingStoragePool[]", "sms.version.version5", F_OPTIONAL)])
CreateDataType("sms.storage.DatastorePair", "DatastorePair", "vmodl.DynamicData", "sms.version.version3", [("datastore1", "vim.Datastore", "sms.version.version3", 0), ("datastore2", "vim.Datastore", "sms.version.version3", 0)])
CreateDataType("sms.storage.DatastoreSet", "DatastoreSet", "vmodl.DynamicData", "sms.version.version5", [("datastore", "vim.Datastore[]", "sms.version.version5", 0)])
CreateDataType("sms.storage.DrsMigrationCapabilityResult", "DrsMigrationCapabilityResult", "vmodl.DynamicData", "sms.version.version3", [("recommendedDatastorePair", "sms.storage.DatastorePair[]", "sms.version.version3", F_OPTIONAL), ("nonRecommendedDatastorePair", "sms.storage.DatastorePair[]", "sms.version.version3", F_OPTIONAL)])
CreateDataType("sms.storage.DrsMigrationCapabilityResultEx", "DrsMigrationCapabilityResultEx", "vmodl.DynamicData", "sms.version.version5", [("nonRecommendedDatastoreSet", "sms.storage.DatastoreSet[]", "sms.version.version5", F_OPTIONAL)])
CreateEnumType("sms.storage.EntityType", "SmsEntityType", "sms.version.version11", ["StorageArrayEntity", "StorageProcessorEntity", "StoragePortEntity", "StorageLunEntity", "StorageFileSystemEntity", "StorageCapabilityEntity", "CapabilitySchemaEntity", "CapabilityProfileEntity", "DefaultProfileEntity", "ResourceAssociationEntity", "StorageContainerEntity", "StorageObjectEntity", "MessageCatalogEntity", "ProtocolEndpointEntity", "VirtualVolumeInfoEntity", "BackingStoragePoolEntity", "FaultDomainEntity", "ReplicationGroupEntity"])
CreateDataType("sms.storage.FaultDomainProviderMapping", "FaultDomainProviderMapping", "vmodl.DynamicData", "sms.version.version11", [("activeProvider", "sms.provider.Provider", "sms.version.version11", 0), ("faultDomainId", "vim.vm.replication.FaultDomainId[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.FileSystemInfo", "StorageFileSystemInfo", "vmodl.DynamicData", "sms.version.version2", [("fileServerName", "string", "sms.version.version2", 0), ("fileSystemPath", "string", "sms.version.version2", 0), ("ipAddress", "string", "sms.version.version2", F_OPTIONAL)])
CreateDataType("sms.storage.LunHbaAssociation", "LunHbaAssociation", "vmodl.DynamicData", "sms.version.version2", [("canonicalName", "string", "sms.version.version2", 0), ("hba", "vim.host.HostBusAdapter[]", "sms.version.version2", 0)])
CreateDataType("sms.storage.NameValuePair", "NameValuePair", "vmodl.DynamicData", "sms.version.version11", [("parameterName", "string", "sms.version.version11", 0), ("parameterValue", "string", "sms.version.version11", 0)])
CreateDataType("sms.storage.StorageAlarm", "StorageAlarm", "vmodl.DynamicData", "sms.version.version11", [("alarmId", "long", "sms.version.version11", 0), ("alarmType", "string", "sms.version.version11", 0), ("containerId", "string", "sms.version.version11", F_OPTIONAL), ("objectId", "string", "sms.version.version11", F_OPTIONAL), ("objectType", "string", "sms.version.version11", 0), ("status", "string", "sms.version.version11", 0), ("alarmTimeStamp", "vmodl.DateTime", "sms.version.version11", 0), ("messageId", "string", "sms.version.version11", 0), ("parameterList", "sms.storage.NameValuePair[]", "sms.version.version11", F_OPTIONAL), ("alarmObject", "anyType", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.StorageArray", "StorageArray", "vmodl.DynamicData", "sms.version.version2", [("name", "string", "sms.version.version2", 0), ("uuid", "string", "sms.version.version2", 0), ("vendorId", "string", "sms.version.version2", 0), ("modelId", "string", "sms.version.version2", 0), ("firmware", "string", "sms.version.version2", F_OPTIONAL), ("alternateName", "string[]", "sms.version.version2", F_OPTIONAL), ("supportedBlockInterface", "string[]", "sms.version.version2", F_OPTIONAL), ("supportedFileSystemInterface", "string[]", "sms.version.version2", F_OPTIONAL), ("supportedProfile", "string[]", "sms.version.version3", F_OPTIONAL), ("priority", "int", "sms.version.version4", F_OPTIONAL)])
CreateEnumType("sms.storage.StorageArray.BlockDeviceInterface", "BlockDeviceInterface", "sms.version.version1", ["fc", "iscsi", "fcoe", "otherBlock"])
CreateEnumType("sms.storage.StorageArray.FileSystemInterface", "FileSystemInterface", "sms.version.version1", ["nfs", "otherFileSystem"])
CreateEnumType("sms.storage.StorageArray.VasaProfile", "VasaProfile", "sms.version.version3", ["blockDevice", "fileSystem", "capability", "policy", "object", "statistics", "storageDrsBlockDevice", "storageDrsFileSystem"])
CreateDataType("sms.storage.StorageCapability", "StorageCapability", "vmodl.DynamicData", "sms.version.version2", [("uuid", "string", "sms.version.version2", 0), ("name", "string", "sms.version.version2", 0), ("description", "string", "sms.version.version2", 0)])
CreateDataType("sms.storage.StorageContainer", "StorageContainer", "vmodl.DynamicData", "sms.version.version5", [("uuid", "string", "sms.version.version5", 0), ("name", "string", "sms.version.version5", 0), ("maxVvolSizeInMB", "long", "sms.version.version5", 0), ("providerId", "string[]", "sms.version.version5", 0), ("arrayId", "string[]", "sms.version.version5", 0)])
CreateDataType("sms.storage.StorageContainerResult", "StorageContainerResult", "vmodl.DynamicData", "sms.version.version5", [("storageContainer", "sms.storage.StorageContainer[]", "sms.version.version5", F_OPTIONAL), ("providerInfo", "sms.provider.ProviderInfo[]", "sms.version.version5", F_OPTIONAL)])
CreateDataType("sms.storage.StorageContainerSpec", "StorageContainerSpec", "vmodl.DynamicData", "sms.version.version5", [("containerId", "string[]", "sms.version.version5", F_OPTIONAL)])
CreateDataType("sms.storage.StorageFileSystem", "StorageFileSystem", "vmodl.DynamicData", "sms.version.version2", [("uuid", "string", "sms.version.version2", 0), ("info", "sms.storage.FileSystemInfo[]", "sms.version.version2", 0), ("nativeSnapshotSupported", "boolean", "sms.version.version2", 0), ("thinProvisioningStatus", "string", "sms.version.version2", 0), ("type", "string", "sms.version.version2", 0), ("version", "string", "sms.version.version2", 0), ("backingConfig", "sms.storage.BackingConfig", "sms.version.version5", F_OPTIONAL)])
CreateEnumType("sms.storage.StorageFileSystem.FileSystemInterfaceVersion", "FileSystemInterfaceVersion", "sms.version.version1", ["NFSV3_0"])
CreateDataType("sms.storage.StorageLun", "StorageLun", "vmodl.DynamicData", "sms.version.version2", [("uuid", "string", "sms.version.version2", 0), ("vSphereLunIdentifier", "string", "sms.version.version2", 0), ("vendorDisplayName", "string", "sms.version.version2", 0), ("capacityInMB", "long", "sms.version.version2", 0), ("usedSpaceInMB", "long", "sms.version.version2", 0), ("lunThinProvisioned", "boolean", "sms.version.version2", 0), ("alternateIdentifier", "string[]", "sms.version.version2", F_OPTIONAL), ("drsManagementPermitted", "boolean", "sms.version.version2", 0), ("thinProvisioningStatus", "string", "sms.version.version2", 0), ("backingConfig", "sms.storage.BackingConfig", "sms.version.version5", F_OPTIONAL)])
CreateDataType("sms.storage.StoragePort", "StoragePort", "vmodl.DynamicData", "sms.version.version2", [("uuid", "string", "sms.version.version2", 0), ("type", "string", "sms.version.version2", 0), ("alternateName", "string[]", "sms.version.version2", F_OPTIONAL)])
CreateDataType("sms.storage.StorageProcessor", "StorageProcessor", "vmodl.DynamicData", "sms.version.version2", [("uuid", "string", "sms.version.version2", 0), ("alternateIdentifer", "string[]", "sms.version.version2", F_OPTIONAL)])
CreateEnumType("sms.storage.ThinProvisioningStatus", "ThinProvisioningStatus", "sms.version.version2", ["RED", "YELLOW", "GREEN"])
CreateDataType("sms.storage.VvolRebindSpec", "VvolRebindSpec", "vmodl.DynamicData", "sms.version.version5", [("storageContainerId", "string", "sms.version.version5", 0), ("vvolId", "string[]", "sms.version.version5", 0)])
CreateDataType("sms.storage.replication.DeviceId", "DeviceId", "vmodl.DynamicData", "sms.version.version11", None)
CreateDataType("sms.storage.replication.FailoverParam", "FailoverParam", "vmodl.DynamicData", "sms.version.version11", [("isPlanned", "boolean", "sms.version.version11", 0), ("checkOnly", "boolean", "sms.version.version11", 0), ("replicationGroupsToFailover", "sms.storage.replication.FailoverParam.ReplicationGroupData[]", "sms.version.version11", F_OPTIONAL), ("policyAssociations", "sms.storage.replication.FailoverParam.PolicyAssociation[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.FailoverParam.ReplicationGroupData", "ReplicationGroupData", "vmodl.DynamicData", "sms.version.version11", [("groupId", "vim.vm.replication.ReplicationGroupId", "sms.version.version11", 0), ("pitId", "sms.storage.replication.PointInTimeReplicaId", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.FailoverParam.PolicyAssociation", "PolicyAssociation", "vmodl.DynamicData", "sms.version.version11", [("id", "sms.storage.replication.DeviceId", "sms.version.version11", 0), ("policyId", "string", "sms.version.version11", 0), ("datastore", "vim.Datastore", "sms.version.version11", 0)])
CreateDataType("sms.storage.replication.FaultDomainInfo", "FaultDomainInfo", "vim.vm.replication.FaultDomainId", "sms.version.version11", [("name", "string", "sms.version.version11", F_OPTIONAL), ("description", "string", "sms.version.version11", F_OPTIONAL), ("storageArrayId", "string", "sms.version.version11", F_OPTIONAL), ("children", "vim.vm.replication.FaultDomainId[]", "sms.version.version11", F_OPTIONAL), ("provider", "sms.provider.Provider", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.GroupInfo", "GroupInfo", "vmodl.DynamicData", "sms.version.version11", [("groupId", "vim.vm.replication.ReplicationGroupId", "sms.version.version11", 0)])
CreateDataType("sms.storage.replication.GroupOperationResult", "GroupOperationResult", "vmodl.DynamicData", "sms.version.version11", [("groupId", "vim.vm.replication.ReplicationGroupId", "sms.version.version11", 0), ("warning", "vmodl.MethodFault[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.PointInTimeReplicaId", "PointInTimeReplicaId", "vmodl.DynamicData", "sms.version.version11", [("id", "string", "sms.version.version11", 0)])
CreateDataType("sms.storage.replication.PromoteParam", "PromoteParam", "vmodl.DynamicData", "sms.version.version11", [("isPlanned", "boolean", "sms.version.version11", 0), ("replicationGroupsToPromote", "vim.vm.replication.ReplicationGroupId[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.QueryPointInTimeReplicaParam", "QueryPointInTimeReplicaParam", "vmodl.DynamicData", "sms.version.version11", [("replicaTimeQueryParam", "sms.storage.replication.QueryPointInTimeReplicaParam.ReplicaQueryIntervalParam", "sms.version.version11", F_OPTIONAL), ("pitName", "string", "sms.version.version11", F_OPTIONAL), ("tags", "string[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.QueryPointInTimeReplicaParam.ReplicaQueryIntervalParam", "ReplicaQueryIntervalParam", "vmodl.DynamicData", "sms.version.version11", [("fromDate", "vmodl.DateTime", "sms.version.version11", F_OPTIONAL), ("toDate", "vmodl.DateTime", "sms.version.version11", F_OPTIONAL), ("number", "int", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.QueryPointInTimeReplicaSuccessResult", "QueryPointInTimeReplicaSuccessResult", "sms.storage.replication.GroupOperationResult", "sms.version.version11", [("replicaInfo", "sms.storage.replication.QueryPointInTimeReplicaSuccessResult.PointInTimeReplicaInfo[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.QueryPointInTimeReplicaSuccessResult.PointInTimeReplicaInfo", "PointInTimeReplicaInfo", "vmodl.DynamicData", "sms.version.version11", [("id", "sms.storage.replication.PointInTimeReplicaId", "sms.version.version11", 0), ("pitName", "string", "sms.version.version11", 0), ("timeStamp", "vmodl.DateTime", "sms.version.version11", 0), ("tags", "string[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.QueryPointInTimeReplicaSummaryResult", "QueryPointInTimeReplicaSummaryResult", "sms.storage.replication.GroupOperationResult", "sms.version.version11", [("intervalResults", "sms.storage.replication.QueryPointInTimeReplicaSummaryResult.ReplicaIntervalQueryResult[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.QueryPointInTimeReplicaSummaryResult.ReplicaIntervalQueryResult", "ReplicaIntervalQueryResult", "vmodl.DynamicData", "sms.version.version11", [("fromDate", "vmodl.DateTime", "sms.version.version11", 0), ("toDate", "vmodl.DateTime", "sms.version.version11", 0), ("number", "int", "sms.version.version11", 0)])
CreateDataType("sms.storage.replication.QueryReplicationGroupSuccessResult", "QueryReplicationGroupSuccessResult", "sms.storage.replication.GroupOperationResult", "sms.version.version11", [("rgInfo", "sms.storage.replication.GroupInfo", "sms.version.version11", 0)])
CreateDataType("sms.storage.replication.QueryReplicationPeerResult", "QueryReplicationPeerResult", "vmodl.DynamicData", "sms.version.version11", [("sourceDomain", "vim.vm.replication.FaultDomainId", "sms.version.version11", 0), ("targetDomain", "vim.vm.replication.FaultDomainId[]", "sms.version.version11", F_OPTIONAL), ("error", "vmodl.MethodFault[]", "sms.version.version11", F_OPTIONAL), ("warning", "vmodl.MethodFault[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.ReplicaId", "ReplicaId", "vmodl.DynamicData", "sms.version.version11", [("id", "string", "sms.version.version11", 0)])
CreateEnumType("sms.storage.replication.ReplicationState", "ReplicationReplicationState", "sms.version.version11", ["SOURCE", "TARGET", "FAILEDOVER", "INTEST", "REMOTE_FAILEDOVER"])
CreateDataType("sms.storage.replication.ReverseReplicationSuccessResult", "ReverseReplicationSuccessResult", "sms.storage.replication.GroupOperationResult", "sms.version.version11", [("newGroupId", "vim.vm.replication.DeviceGroupId", "sms.version.version11", 0)])
CreateDataType("sms.storage.replication.SourceGroupInfo", "SourceGroupInfo", "sms.storage.replication.GroupInfo", "sms.version.version11", [("name", "string", "sms.version.version11", F_OPTIONAL), ("description", "string", "sms.version.version11", F_OPTIONAL), ("state", "string", "sms.version.version11", 0), ("replica", "sms.storage.replication.SourceGroupInfo.ReplicationTargetInfo[]", "sms.version.version11", F_OPTIONAL), ("memberInfo", "sms.storage.replication.SourceGroupMemberInfo[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.SourceGroupInfo.ReplicationTargetInfo", "ReplicationTargetInfo", "vmodl.DynamicData", "sms.version.version11", [("targetGroupId", "vim.vm.replication.ReplicationGroupId", "sms.version.version11", 0), ("replicationAgreementDescription", "string", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.SourceGroupMemberInfo", "SourceGroupMemberInfo", "vmodl.DynamicData", "sms.version.version11", [("deviceId", "sms.storage.replication.DeviceId", "sms.version.version11", 0), ("targetId", "sms.storage.replication.SourceGroupMemberInfo.TargetDeviceId[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.SourceGroupMemberInfo.TargetDeviceId", "TargetDeviceId", "vmodl.DynamicData", "sms.version.version11", [("domainId", "vim.vm.replication.FaultDomainId", "sms.version.version11", 0), ("deviceId", "sms.storage.replication.ReplicaId", "sms.version.version11", 0)])
CreateDataType("sms.storage.replication.SyncReplicationGroupSuccessResult", "SyncReplicationGroupSuccessResult", "sms.storage.replication.GroupOperationResult", "sms.version.version11", [("timeStamp", "vmodl.DateTime", "sms.version.version11", 0), ("pitId", "sms.storage.replication.PointInTimeReplicaId", "sms.version.version11", F_OPTIONAL), ("pitName", "string", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.TargetGroupInfo", "TargetGroupInfo", "sms.storage.replication.GroupInfo", "sms.version.version11", [("sourceInfo", "sms.storage.replication.TargetGroupInfo.TargetToSourceInfo", "sms.version.version11", 0), ("state", "string", "sms.version.version11", 0), ("devices", "sms.storage.replication.TargetGroupMemberInfo[]", "sms.version.version11", F_OPTIONAL), ("isPromoteCapable", "boolean", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.TargetGroupInfo.TargetToSourceInfo", "TargetToSourceInfo", "vmodl.DynamicData", "sms.version.version11", [("sourceGroupId", "vim.vm.replication.ReplicationGroupId", "sms.version.version11", 0), ("replicationAgreementDescription", "string", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.TargetGroupMemberInfo", "TargetGroupMemberInfo", "vmodl.DynamicData", "sms.version.version11", [("replicaId", "sms.storage.replication.ReplicaId", "sms.version.version11", 0), ("sourceId", "sms.storage.replication.DeviceId", "sms.version.version11", 0), ("targetDatastore", "vim.Datastore", "sms.version.version11", 0)])
CreateDataType("sms.storage.replication.TestFailoverParam", "TestFailoverParam", "sms.storage.replication.FailoverParam", "sms.version.version11", None)
CreateDataType("sms.storage.replication.VVolId", "VVolId", "sms.storage.replication.DeviceId", "sms.version.version11", [("id", "string", "sms.version.version11", 0)])
CreateDataType("sms.storage.replication.VirtualDiskId", "VasaVirtualDiskId", "sms.storage.replication.DeviceId", "sms.version.version11", [("diskId", "string", "sms.version.version11", 0)])
CreateDataType("sms.storage.replication.VirtualDiskKey", "VirtualDiskKey", "sms.storage.replication.DeviceId", "sms.version.version11", [("vmInstanceUUID", "string", "sms.version.version11", 0), ("deviceKey", "int", "sms.version.version11", 0)])
CreateDataType("sms.storage.replication.VirtualDiskMoId", "VirtualDiskMoId", "sms.storage.replication.DeviceId", "sms.version.version12", [("vcUuid", "string", "sms.version.version12", F_OPTIONAL), ("vmMoid", "string", "sms.version.version12", 0), ("diskKey", "string", "sms.version.version12", 0)])
CreateDataType("sms.storage.replication.VirtualMachineId", "VirtualMachineId", "sms.storage.replication.DeviceId", "sms.version.version11", None)
CreateDataType("sms.storage.replication.VirtualMachineMoId", "VirtualMachineMoId", "sms.storage.replication.VirtualMachineId", "sms.version.version12", [("vcUuid", "string", "sms.version.version12", F_OPTIONAL), ("vmMoid", "string", "sms.version.version12", 0)])
CreateDataType("sms.storage.replication.VirtualMachineUUID", "VirtualMachineUUID", "sms.storage.replication.VirtualMachineId", "sms.version.version11", [("vmInstanceUUID", "string", "sms.version.version11", 0)])
CreateDataType("sms.fault.CertificateAuthorityFault", "CertificateAuthorityFault", "sms.fault.ProviderRegistrationFault", "sms.version.version5", [("faultCode", "int", "sms.version.version5", 0)])
CreateDataType("sms.fault.CertificateNotImported", "CertificateNotImported", "sms.fault.ProviderRegistrationFault", "sms.version.version5", None)
CreateDataType("sms.fault.CertificateNotTrusted", "CertificateNotTrusted", "sms.fault.ProviderRegistrationFault", "sms.version.version2", [("certificate", "string", "sms.version.version2", 0)])
CreateDataType("sms.fault.IncorrectUsernamePassword", "IncorrectUsernamePassword", "sms.fault.ProviderRegistrationFault", "sms.version.version2", None)
CreateDataType("sms.fault.InvalidCertificate", "InvalidCertificate", "sms.fault.ProviderRegistrationFault", "sms.version.version4", [("certificate", "string", "sms.version.version4", 0)])
CreateDataType("sms.fault.InvalidUrl", "InvalidUrl", "sms.fault.ProviderRegistrationFault", "sms.version.version4", [("url", "string", "sms.version.version4", 0)])
CreateDataType("sms.fault.NoCommonProviderForAllBackings", "NoCommonProviderForAllBackings", "sms.fault.QueryExecutionFault", "sms.version.version2", None)
CreateDataType("sms.fault.ProviderNotFound", "ProviderNotFound", "sms.fault.QueryExecutionFault", "sms.version.version2", None)
CreateDataType("sms.fault.VmscProfileNotSupported", "VmscProfileNotSupported", "sms.fault.VvolRebindFault", "sms.version.version5", None)
CreateDataType("sms.fault.replication.AlreadyDone", "AlreadyDone", "sms.fault.replication.ReplicationFault", "sms.version.version11", None)
CreateDataType("sms.fault.replication.InvalidFunctionTarget", "InvalidFunctionTarget", "sms.fault.replication.ReplicationFault", "sms.version.version11", None)
CreateDataType("sms.fault.replication.InvalidReplicationState", "InvalidReplicationState", "sms.fault.replication.ReplicationFault", "sms.version.version11", [("desiredState", "string[]", "sms.version.version11", F_OPTIONAL), ("currentState", "string", "sms.version.version11", 0)])
CreateDataType("sms.fault.replication.NoReplicationTarget", "NoReplicationTarget", "sms.fault.replication.ReplicationFault", "sms.version.version11", None)
CreateDataType("sms.fault.replication.NoValidReplica", "NoValidReplica", "sms.fault.replication.ReplicationFault", "sms.version.version11", [("deviceId", "sms.storage.replication.DeviceId", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.fault.replication.PeerNotReachable", "PeerNotReachable", "sms.fault.replication.ReplicationFault", "sms.version.version11", None)
CreateDataType("sms.storage.FcStoragePort", "FcStoragePort", "sms.storage.StoragePort", "sms.version.version2", [("portWwn", "string", "sms.version.version2", 0), ("nodeWwn", "string", "sms.version.version2", 0)])
CreateDataType("sms.storage.FcoeStoragePort", "FcoeStoragePort", "sms.storage.StoragePort", "sms.version.version2", [("portWwn", "string", "sms.version.version2", 0), ("nodeWwn", "string", "sms.version.version2", 0)])
CreateDataType("sms.storage.IscsiStoragePort", "IscsiStoragePort", "sms.storage.StoragePort", "sms.version.version2", [("identifier", "string", "sms.version.version2", 0)])
CreateDataType("sms.storage.replication.FailoverSuccessResult", "FailoverSuccessResult", "sms.storage.replication.GroupOperationResult", "sms.version.version11", [("newState", "string", "sms.version.version11", 0), ("pitId", "sms.storage.replication.PointInTimeReplicaId", "sms.version.version11", F_OPTIONAL), ("pitIdBeforeFailover", "sms.storage.replication.PointInTimeReplicaId", "sms.version.version11", F_OPTIONAL), ("recoveredDeviceInfo", "sms.storage.replication.FailoverSuccessResult.RecoveredDevice[]", "sms.version.version11", F_OPTIONAL), ("timeStamp", "vmodl.DateTime", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.FailoverSuccessResult.RecoveredDiskInfo", "RecoveredDiskInfo", "vmodl.DynamicData", "sms.version.version11", [("deviceKey", "int", "sms.version.version11", 0), ("dsUrl", "string", "sms.version.version11", 0), ("diskPath", "string", "sms.version.version11", 0)])
CreateDataType("sms.storage.replication.FailoverSuccessResult.RecoveredDevice", "RecoveredDevice", "vmodl.DynamicData", "sms.version.version11", [("targetDeviceId", "sms.storage.replication.ReplicaId", "sms.version.version11", F_OPTIONAL), ("recoveredDeviceId", "sms.storage.replication.DeviceId", "sms.version.version11", F_OPTIONAL), ("sourceDeviceId", "sms.storage.replication.DeviceId", "sms.version.version11", 0), ("info", "string[]", "sms.version.version11", F_OPTIONAL), ("datastore", "vim.Datastore", "sms.version.version11", 0), ("recoveredDiskInfo", "sms.storage.replication.FailoverSuccessResult.RecoveredDiskInfo[]", "sms.version.version11", F_OPTIONAL), ("error", "vmodl.MethodFault", "sms.version.version11", F_OPTIONAL), ("warnings", "vmodl.MethodFault[]", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.GroupErrorResult", "GroupErrorResult", "sms.storage.replication.GroupOperationResult", "sms.version.version11", [("error", "vmodl.MethodFault[]", "sms.version.version11", 0)])
CreateDataType("sms.storage.replication.RecoveredTargetGroupMemberInfo", "RecoveredTargetGroupMemberInfo", "sms.storage.replication.TargetGroupMemberInfo", "sms.version.version11", [("recoveredDeviceId", "sms.storage.replication.DeviceId", "sms.version.version11", F_OPTIONAL)])
CreateDataType("sms.storage.replication.VirtualMachineFilePath", "VirtualMachineFilePath", "sms.storage.replication.VirtualMachineId", "sms.version.version11", [("vcUuid", "string", "sms.version.version11", F_OPTIONAL), ("dsUrl", "string", "sms.version.version11", 0), ("vmxPath", "string", "sms.version.version11", 0)])



# VMware vSphere Python SDK
# Copyright (c) 2016 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

## @file task.py
## @brief Task functions
##
## This module provies synchronization of client/server operations
## since many VIM operations return 'tasks' which can have
## varying completion times.

"""
Task functions

This module provies synchronization of client/server operations since
many VIM operations return 'tasks' which can have varying completion
times.
"""

from pyVmomi import vmodl, vim


##
## @brief Exception class to represent when task is blocked (e.g.:
## waiting for an answer to a question.
##
class TaskBlocked(Exception):
    """
    Exception class to represent when task is blocked (e.g.: waiting
    for an answer to a question.
    """
    pass


#
# TaskUpdates
#     verbose information about task progress
#
def TaskUpdatesVerbose(task, progress):
    if isinstance(task.info.progress, int):
        info = task.info
        if not isinstance(progress, str):
            progress = '%d%% (%s)' % (info.progress, info.state)
        print('Task %s (key:%s, desc:%s) - %s' % (
            info.name.info.name, info.key, info.description, progress))


globalTaskUpdate = None


def SetTasksVerbose(verbose=True):
    global globalTaskUpdate
    if verbose:
        globalTaskUpdate = TaskUpdatesVerbose
    else:
        globalTaskUpdate = None


##
## @param raiseOnError [in] Any exception thrown is thrown up to the caller if
## raiseOnError is set to true
## @param si [in] ServiceInstance to use. If set to None, use the default one.
## @param pc [in] property collector to use else retrieve one from cache
## @param onProgressUpdate [in] callable to call with task progress updates.
##    For example:
##
##    def OnTaskProgressUpdate(task, percentDone):
##       sys.stderr.write('# Task %s: %d%% complete ...\n' % (task, percentDone))
##
## Given a task object and a service instance, wait for the task completion
##
## @return state as either "success" or "error". To look at any errors, the
## user should reexamine the task object.
##
## NOTE: This is a blocking call.
##
def WaitForTask(task,
                raiseOnError=True,
                si=None,
                pc=None,
                onProgressUpdate=None):
    """
    Wait for task to complete.

    @type  raiseOnError      : bool
    @param raiseOnError      : Any exception thrown is thrown up to the caller
                               if raiseOnError is set to true.
    @type  si                : ManagedObjectReference to a ServiceInstance.
    @param si                : ServiceInstance to use. If None, use the
                               information from the task.
    @type  pc                : ManagedObjectReference to a PropertyCollector.
    @param pc                : Property collector to use. If None, get it from
                               the ServiceInstance.
    @type  onProgressUpdate  : callable
    @param onProgressUpdate  : Callable to call with task progress updates.

        For example::

            def OnTaskProgressUpdate(task, percentDone):
                print 'Task %s is %d%% complete.' % (task, percentDone)
    """

    if si is None:
        si = vim.ServiceInstance("ServiceInstance", task._stub)
    if pc is None:
        pc = si.content.propertyCollector

    progressUpdater = ProgressUpdater(task, onProgressUpdate)
    progressUpdater.Update('created')

    filter = CreateFilter(pc, task)

    version, state = None, None
    # Loop looking for updates till the state moves to a completed state.
    while state not in (vim.TaskInfo.State.success, vim.TaskInfo.State.error):
        try:
            version, state = GetTaskStatus(task, version, pc)
            progressUpdater.UpdateIfNeeded()
        except vmodl.fault.ManagedObjectNotFound as e:
            print("Task object has been deleted: %s" % e.obj)
            break

    filter.Destroy()

    if state == "error":
        progressUpdater.Update('error: %s' % str(task.info.error))
        if raiseOnError:
            raise task.info.error
        else:
            print("Task reported error: " + str(task.info.error))
    else:
        progressUpdater.Update('completed')

    return state


## Wait for multiple tasks to complete
#  See WaitForTask for detail
#
#  Difference: WaitForTasks won't return the state of tasks. User can check
#  tasks state directly with task.info.state
#
#  TODO: Did not check for question pending
def WaitForTasks(tasks,
                 raiseOnError=True,
                 si=None,
                 pc=None,
                 onProgressUpdate=None,
                 results=None):
    """
    Wait for mulitiple tasks to complete. Much faster than calling WaitForTask
    N times
    """

    if not tasks:
        return

    if si is None:
        si = vim.ServiceInstance("ServiceInstance", tasks[0]._stub)
    if pc is None:
        pc = si.content.propertyCollector
    if results is None:
        results = []

    progressUpdaters = {}
    for task in tasks:
        progressUpdater = ProgressUpdater(task, onProgressUpdate)
        progressUpdater.Update('created')
        progressUpdaters[str(task)] = progressUpdater

    filter = CreateTasksFilter(pc, tasks)

    try:
        version, state = None, None

        # Loop looking for updates till the state moves to a completed state.
        while len(progressUpdaters):
            update = pc.WaitForUpdates(version)
            for filterSet in update.filterSet:
                for objSet in filterSet.objectSet:
                    task = objSet.obj
                    taskId = str(task)
                    for change in objSet.changeSet:
                        if change.name == 'info':
                            state = change.val.state
                        elif change.name == 'info.state':
                            state = change.val
                        else:
                            continue

                        progressUpdater = progressUpdaters.get(taskId)
                        if not progressUpdater:
                            continue

                        if state == vim.TaskInfo.State.success:
                            progressUpdater.Update('completed')
                            progressUpdaters.pop(taskId)
                            # cache the results, as task objects could expire if one
                            # of the tasks take a longer time to complete
                            results.append(task.info.result)
                        elif state == vim.TaskInfo.State.error:
                            err = task.info.error
                            progressUpdater.Update('error: %s' % str(err))
                            if raiseOnError:
                                raise err
                            else:
                                print("Task %s reported error: %s" % (taskId, str(err)))
                                progressUpdaters.pop(taskId)
                        else:
                            if onProgressUpdate:
                                progressUpdater.UpdateIfNeeded()
            # Move to next version
            version = update.version
    finally:
        if filter:
            filter.Destroy()
    return


def GetTaskStatus(task, version, pc):
    update = pc.WaitForUpdates(version)
    state = task.info.state

    if (state == 'running' and task.info.name is not None and task.info.name.info.name != "Destroy"
        and task.info.name.info.name != "Relocate"):
        CheckForQuestionPending(task)

    return update.version, state


def CreateFilter(pc, task):
    """ Create property collector filter for task """
    return CreateTasksFilter(pc, [task])


def CreateTasksFilter(pc, tasks):
    """ Create property collector filter for tasks """
    if not tasks:
        return None

    # First create the object specification as the task object.
    objspecs = [vmodl.query.PropertyCollector.ObjectSpec(obj=task)
                for task in tasks]

    # Next, create the property specification as the state.
    propspec = vmodl.query.PropertyCollector.PropertySpec(
        type=vim.Task, pathSet=[], all=True)

    # Create a filter spec with the specified object and property spec.
    filterspec = vmodl.query.PropertyCollector.FilterSpec()
    filterspec.objectSet = objspecs
    filterspec.propSet = [propspec]

    # Create the filter
    return pc.CreateFilter(filterspec, True)


def CheckForQuestionPending(task):
    """
    Check to see if VM needs to ask a question, throw exception
    """

    vm = task.info.entity
    if vm is not None and isinstance(vm, vim.VirtualMachine):
        qst = vm.runtime.question
        if qst is not None:
            raise TaskBlocked("Task blocked, User Intervention required")


##
## @brief Class that keeps track of task percentage complete and calls
## a provided callback when it changes.
##
class ProgressUpdater(object):
    """
    Class that keeps track of task percentage complete and calls a
    provided callback when it changes.
    """

    def __init__(self, task, onProgressUpdate):
        self.task = task
        self.onProgressUpdate = onProgressUpdate
        self.prevProgress = 0
        self.progress = 0

    def Update(self, state):
        global globalTaskUpdate
        taskUpdate = globalTaskUpdate
        if self.onProgressUpdate:
            taskUpdate = self.onProgressUpdate
        if taskUpdate:
            taskUpdate(self.task, state)

    def UpdateIfNeeded(self):
        self.progress = self.task.info.progress

        if self.progress != self.prevProgress:
            self.Update(self.progress)

        self.prevProgress = self.progress



# VMware vSphere Python SDK
# Copyright (c) 2008-2015 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import print_function

import tests

from pyVim import connect


class ManagedObjectTests(tests.VCRTestBase):

    @tests.VCRTestBase.my_vcr.use_cassette('root_folder_parent.yaml',
                      cassette_library_dir=tests.fixtures_path,
                      record_mode='once')
    def test_root_folder_parent(self):
        # see: http://python3porting.com/noconv.html
        si = connect.SmartConnect(host='vcsa',
                                  user='my_user',
                                  pwd='my_password')

        root_folder = si.content.rootFolder
        self.assertTrue(hasattr(root_folder, 'parent'))
        # NOTE (hartsock): assertIsNone does not work in Python 2.6
        self.assertTrue(root_folder.parent is None)



# VMware vSphere Python SDK
# Copyright (c) 2008-2015 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import absolute_import
from pyVmomi.VmomiSupport import nsMap, versionMap, versionIdMap, serviceNsMap, parentMap

## Add an API version
def AddVersion(version, ns, versionId='', isLegacy=0, serviceNs=''):
  if not ns:
     ns = serviceNs
  if not (version in parentMap):
      nsMap[version] = ns
      if len(versionId) > 0:
         versionMap[ns + '/' + versionId] = version
      if isLegacy or ns is "":
         versionMap[ns] = version
      versionIdMap[version] = versionId
      if not serviceNs:
         serviceNs = ns
      serviceNsMap[version] = serviceNs
      parentMap[version] = {}

## Check if a version is a child of another
def IsChildVersion(child, parent):
   return child == parent or parent in parentMap[child]



# ******* WARNING - AUTO GENERATED CODE - DO NOT EDIT *******
from __future__ import absolute_import
from pyVmomi.VmomiSupport import CreateDataType, CreateManagedType, CreateEnumType, AddVersion, AddVersionParent, F_LINK, F_LINKABLE, F_OPTIONAL, F_SECRET
from pyVmomi.VmomiSupport import newestVersions, stableVersions, publicVersions, oldestVersions

AddVersion("vmodl.query.version.version1", "", "", 0, "vim25")
AddVersion("vmodl.query.version.version2", "", "", 0, "vim25")
AddVersion("vmodl.query.version.version3", "", "", 0, "vim25")
AddVersion("vmodl.query.version.version4", "", "", 0, "vim25")
AddVersion("vmodl.version.version2", "", "", 0, "vim25")
AddVersion("vmodl.version.version1", "", "", 0, "vim25")
AddVersion("vmodl.version.version0", "", "", 0, "vim25")
AddVersionParent("vmodl.query.version.version1", "vmodl.query.version.version1")
AddVersionParent("vmodl.query.version.version1", "vmodl.version.version0")
AddVersionParent("vmodl.query.version.version2", "vmodl.query.version.version1")
AddVersionParent("vmodl.query.version.version2", "vmodl.query.version.version2")
AddVersionParent("vmodl.query.version.version2", "vmodl.version.version1")
AddVersionParent("vmodl.query.version.version2", "vmodl.version.version0")
AddVersionParent("vmodl.query.version.version3", "vmodl.query.version.version1")
AddVersionParent("vmodl.query.version.version3", "vmodl.query.version.version2")
AddVersionParent("vmodl.query.version.version3", "vmodl.query.version.version3")
AddVersionParent("vmodl.query.version.version3", "vmodl.version.version1")
AddVersionParent("vmodl.query.version.version3", "vmodl.version.version0")
AddVersionParent("vmodl.query.version.version4", "vmodl.query.version.version1")
AddVersionParent("vmodl.query.version.version4", "vmodl.query.version.version2")
AddVersionParent("vmodl.query.version.version4", "vmodl.query.version.version3")
AddVersionParent("vmodl.query.version.version4", "vmodl.query.version.version4")
AddVersionParent("vmodl.query.version.version4", "vmodl.version.version2")
AddVersionParent("vmodl.query.version.version4", "vmodl.version.version1")
AddVersionParent("vmodl.query.version.version4", "vmodl.version.version0")
AddVersionParent("vmodl.version.version2", "vmodl.version.version2")
AddVersionParent("vmodl.version.version2", "vmodl.version.version1")
AddVersionParent("vmodl.version.version2", "vmodl.version.version0")
AddVersionParent("vmodl.version.version1", "vmodl.version.version1")
AddVersionParent("vmodl.version.version1", "vmodl.version.version0")
AddVersionParent("vmodl.version.version0", "vmodl.version.version0")

newestVersions.Add("vmodl.query.version.version4")
stableVersions.Add("vmodl.query.version.version4")
publicVersions.Add("vmodl.query.version.version4")
oldestVersions.Add("vmodl.query.version.version1")

CreateDataType("vmodl.query.InvalidCollectorVersion", "InvalidCollectorVersion", "vmodl.MethodFault", "vmodl.query.version.version1", None)
CreateDataType("vmodl.query.InvalidProperty", "InvalidProperty", "vmodl.MethodFault", "vmodl.query.version.version1", [("name", "vmodl.PropertyPath", "vmodl.query.version.version1", 0)])
CreateManagedType("vmodl.query.PropertyCollector", "PropertyCollector", "vmodl.ManagedObject", "vmodl.query.version.version1", [("filter", "vmodl.query.PropertyCollector.Filter[]", "vmodl.query.version.version1", F_OPTIONAL, "System.View")], [("createFilter", "CreateFilter", "vmodl.query.version.version1", (("spec", "vmodl.query.PropertyCollector.FilterSpec", "vmodl.query.version.version1", 0, None),("partialUpdates", "boolean", "vmodl.query.version.version1", 0, None),), (0, "vmodl.query.PropertyCollector.Filter", "vmodl.query.PropertyCollector.Filter"), "System.View", ["vmodl.query.InvalidProperty", ]), ("retrieveContents", "RetrieveProperties", "vmodl.query.version.version1", (("specSet", "vmodl.query.PropertyCollector.FilterSpec[]", "vmodl.query.version.version1", 0, None),), (F_OPTIONAL, "vmodl.query.PropertyCollector.ObjectContent[]", "vmodl.query.PropertyCollector.ObjectContent[]"), "System.Anonymous", ["vmodl.query.InvalidProperty", ]), ("checkForUpdates", "CheckForUpdates", "vmodl.query.version.version1", (("version", "string", "vmodl.query.version.version1", F_OPTIONAL, None),), (F_OPTIONAL, "vmodl.query.PropertyCollector.UpdateSet", "vmodl.query.PropertyCollector.UpdateSet"), "System.View", ["vmodl.query.InvalidCollectorVersion", ]), ("waitForUpdates", "WaitForUpdates", "vmodl.query.version.version1", (("version", "string", "vmodl.query.version.version1", F_OPTIONAL, None),), (0, "vmodl.query.PropertyCollector.UpdateSet", "vmodl.query.PropertyCollector.UpdateSet"), "System.View", ["vmodl.query.InvalidCollectorVersion", ]), ("cancelWaitForUpdates", "CancelWaitForUpdates", "vmodl.query.version.version1", (), (0, "void", "void"), "System.View", None), ("waitForUpdatesEx", "WaitForUpdatesEx", "vmodl.query.version.version3", (("version", "string", "vmodl.query.version.version3", F_OPTIONAL, None),("options", "vmodl.query.PropertyCollector.WaitOptions", "vmodl.query.version.version3", F_OPTIONAL, None),), (F_OPTIONAL, "vmodl.query.PropertyCollector.UpdateSet", "vmodl.query.PropertyCollector.UpdateSet"), "System.View", ["vmodl.query.InvalidCollectorVersion", ]), ("retrievePropertiesEx", "RetrievePropertiesEx", "vmodl.query.version.version3", (("specSet", "vmodl.query.PropertyCollector.FilterSpec[]", "vmodl.query.version.version3", 0, None),("options", "vmodl.query.PropertyCollector.RetrieveOptions", "vmodl.query.version.version3", 0, None),), (F_OPTIONAL, "vmodl.query.PropertyCollector.RetrieveResult", "vmodl.query.PropertyCollector.RetrieveResult"), "System.Anonymous", ["vmodl.query.InvalidProperty", ]), ("continueRetrievePropertiesEx", "ContinueRetrievePropertiesEx", "vmodl.query.version.version3", (("token", "string", "vmodl.query.version.version3", 0, None),), (0, "vmodl.query.PropertyCollector.RetrieveResult", "vmodl.query.PropertyCollector.RetrieveResult"), "System.Anonymous", ["vmodl.query.InvalidProperty", ]), ("cancelRetrievePropertiesEx", "CancelRetrievePropertiesEx", "vmodl.query.version.version3", (("token", "string", "vmodl.query.version.version3", 0, None),), (0, "void", "void"), "System.Anonymous", ["vmodl.query.InvalidProperty", ]), ("createPropertyCollector", "CreatePropertyCollector", "vmodl.query.version.version3", (), (0, "vmodl.query.PropertyCollector", "vmodl.query.PropertyCollector"), "System.View", None), ("destroy", "DestroyPropertyCollector", "vmodl.query.version.version3", (), (0, "void", "void"), "System.View", None)])
CreateDataType("vmodl.query.PropertyCollector.FilterSpec", "PropertyFilterSpec", "vmodl.DynamicData", "vmodl.query.version.version1", [("propSet", "vmodl.query.PropertyCollector.PropertySpec[]", "vmodl.query.version.version1", 0), ("objectSet", "vmodl.query.PropertyCollector.ObjectSpec[]", "vmodl.query.version.version1", 0), ("reportMissingObjectsInResults", "boolean", "vmodl.query.version.version3", F_OPTIONAL)])
CreateDataType("vmodl.query.PropertyCollector.PropertySpec", "PropertySpec", "vmodl.DynamicData", "vmodl.query.version.version1", [("type", "vmodl.TypeName", "vmodl.query.version.version1", 0), ("all", "boolean", "vmodl.query.version.version1", F_OPTIONAL), ("pathSet", "vmodl.PropertyPath[]", "vmodl.query.version.version1", F_OPTIONAL)])
CreateDataType("vmodl.query.PropertyCollector.ObjectSpec", "ObjectSpec", "vmodl.DynamicData", "vmodl.query.version.version1", [("obj", "vmodl.ManagedObject", "vmodl.query.version.version1", 0), ("skip", "boolean", "vmodl.query.version.version1", F_OPTIONAL), ("selectSet", "vmodl.query.PropertyCollector.SelectionSpec[]", "vmodl.query.version.version1", F_OPTIONAL)])
CreateDataType("vmodl.query.PropertyCollector.SelectionSpec", "SelectionSpec", "vmodl.DynamicData", "vmodl.query.version.version1", [("name", "string", "vmodl.query.version.version1", F_OPTIONAL)])
CreateDataType("vmodl.query.PropertyCollector.TraversalSpec", "TraversalSpec", "vmodl.query.PropertyCollector.SelectionSpec", "vmodl.query.version.version1", [("type", "vmodl.TypeName", "vmodl.query.version.version1", 0), ("path", "vmodl.PropertyPath", "vmodl.query.version.version1", 0), ("skip", "boolean", "vmodl.query.version.version1", F_OPTIONAL), ("selectSet", "vmodl.query.PropertyCollector.SelectionSpec[]", "vmodl.query.version.version1", F_OPTIONAL)])
CreateManagedType("vmodl.query.PropertyCollector.Filter", "PropertyFilter", "vmodl.ManagedObject", "vmodl.query.version.version1", [("spec", "vmodl.query.PropertyCollector.FilterSpec", "vmodl.query.version.version1", 0, None), ("partialUpdates", "boolean", "vmodl.query.version.version1", 0, None)], [("destroy", "DestroyPropertyFilter", "vmodl.query.version.version1", (), (0, "void", "void"), None, None)])
CreateDataType("vmodl.query.PropertyCollector.ObjectContent", "ObjectContent", "vmodl.DynamicData", "vmodl.query.version.version1", [("obj", "vmodl.ManagedObject", "vmodl.query.version.version1", 0), ("propSet", "vmodl.DynamicProperty[]", "vmodl.query.version.version1", F_OPTIONAL), ("missingSet", "vmodl.query.PropertyCollector.MissingProperty[]", "vmodl.query.version.version1", F_OPTIONAL)])
CreateDataType("vmodl.query.PropertyCollector.UpdateSet", "UpdateSet", "vmodl.DynamicData", "vmodl.query.version.version1", [("version", "string", "vmodl.query.version.version1", 0), ("filterSet", "vmodl.query.PropertyCollector.FilterUpdate[]", "vmodl.query.version.version1", F_OPTIONAL), ("truncated", "boolean", "vmodl.query.version.version3", F_OPTIONAL)])
CreateDataType("vmodl.query.PropertyCollector.FilterUpdate", "PropertyFilterUpdate", "vmodl.DynamicData", "vmodl.query.version.version1", [("filter", "vmodl.query.PropertyCollector.Filter", "vmodl.query.version.version1", 0), ("objectSet", "vmodl.query.PropertyCollector.ObjectUpdate[]", "vmodl.query.version.version1", F_OPTIONAL), ("missingSet", "vmodl.query.PropertyCollector.MissingObject[]", "vmodl.query.version.version1", F_OPTIONAL)])
CreateDataType("vmodl.query.PropertyCollector.ObjectUpdate", "ObjectUpdate", "vmodl.DynamicData", "vmodl.query.version.version1", [("kind", "vmodl.query.PropertyCollector.ObjectUpdate.Kind", "vmodl.query.version.version1", 0), ("obj", "vmodl.ManagedObject", "vmodl.query.version.version1", 0), ("changeSet", "vmodl.query.PropertyCollector.Change[]", "vmodl.query.version.version1", F_OPTIONAL), ("missingSet", "vmodl.query.PropertyCollector.MissingProperty[]", "vmodl.query.version.version1", F_OPTIONAL)])
CreateEnumType("vmodl.query.PropertyCollector.ObjectUpdate.Kind", "ObjectUpdateKind", "vmodl.query.version.version1", ["modify", "enter", "leave"])
CreateDataType("vmodl.query.PropertyCollector.Change", "PropertyChange", "vmodl.DynamicData", "vmodl.query.version.version1", [("name", "vmodl.PropertyPath", "vmodl.query.version.version1", 0), ("op", "vmodl.query.PropertyCollector.Change.Op", "vmodl.query.version.version1", 0), ("val", "anyType", "vmodl.query.version.version1", F_OPTIONAL)])
CreateEnumType("vmodl.query.PropertyCollector.Change.Op", "PropertyChangeOp", "vmodl.query.version.version1", ["add", "remove", "assign", "indirectRemove"])
CreateDataType("vmodl.query.PropertyCollector.MissingProperty", "MissingProperty", "vmodl.DynamicData", "vmodl.query.version.version1", [("path", "vmodl.PropertyPath", "vmodl.query.version.version1", 0), ("fault", "vmodl.MethodFault", "vmodl.query.version.version1", 0)])
CreateDataType("vmodl.query.PropertyCollector.MissingObject", "MissingObject", "vmodl.DynamicData", "vmodl.query.version.version1", [("obj", "vmodl.ManagedObject", "vmodl.query.version.version1", 0), ("fault", "vmodl.MethodFault", "vmodl.query.version.version1", 0)])
CreateDataType("vmodl.query.PropertyCollector.WaitOptions", "WaitOptions", "vmodl.DynamicData", "vmodl.query.version.version3", [("maxWaitSeconds", "int", "vmodl.query.version.version3", F_OPTIONAL), ("maxObjectUpdates", "int", "vmodl.query.version.version3", F_OPTIONAL)])
CreateDataType("vmodl.query.PropertyCollector.RetrieveOptions", "RetrieveOptions", "vmodl.DynamicData", "vmodl.query.version.version3", [("maxObjects", "int", "vmodl.query.version.version3", F_OPTIONAL)])
CreateDataType("vmodl.query.PropertyCollector.RetrieveResult", "RetrieveResult", "vmodl.DynamicData", "vmodl.query.version.version3", [("token", "string", "vmodl.query.version.version3", F_OPTIONAL), ("objects", "vmodl.query.PropertyCollector.ObjectContent[]", "vmodl.query.version.version3", 0)])



# VMware vSphere Python SDK
# Copyright (c) 2008-2015 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This module implements the cache decorator
"""
__author__ = "VMware, Inc."

def Cache(fn):
   """ Function cache decorator """
   def fnCache(*args, **kwargs):
      """ Cache function """
      key = (args and tuple(args) or None,
             kwargs and frozenset(kwargs.items()) or None)
      if key not in fn.__cached__:
         fn.__cached__[key] = cache = fn(*args, **kwargs)
      else:
         cache = fn.__cached__[key]
      return cache

   def ResetCache():
      """ Reset cache """
      fn.__cached__ = {}

   setattr(fn, "__cached__", {})
   setattr(fn, "__resetcache__", ResetCache)
   fnCache.__name__ = fn.__name__
   fnCache.__doc__ = fn.__doc__
   fnCache.__dict__.update(fn.__dict__)
   return fnCache



# VMware vSphere Python SDK
# Copyright (c) 2008-2015 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import print_function

import tests

from pyVim import connect
from pyVmomi import vim


class VirtualMachineTests(tests.VCRTestBase):

    @tests.VCRTestBase.my_vcr.use_cassette('vm_nic_data.yaml',
                      cassette_library_dir=tests.fixtures_path,
                      record_mode='never')
    def test_vm_nic_data(self):
        data = {'ESXi-5.5-16': [],
                'ESXi-5.5-17': [],
                'ESXi-5.5-18': [],
                'ESXi11': ['00:0c:29:e1:e0:f8'],
                'ESXi12': ['00:50:56:b4:3c:3c'],
                'ESXi20': ['00:50:56:b4:fc:9b', '00:50:56:b4:28:e7'],
                'ESXi21': ['00:50:56:b4:8d:7a', '00:50:56:b4:39:b8'],
                'ESXi22': ['00:0c:29:36:b5:5a', '00:0c:29:36:b5:64'],
                'ESXi23': ['00:50:56:b4:91:f9', '00:50:56:b4:90:9f'],
                'ESXi38-v5.0': ['00:0c:29:ce:6a:d8', '00:0c:29:ce:6a:e2'],
                'MARVEL-Agents_of_Atlas': [],
                'MARVEL-Alex_Power': [],
                'MARVEL-Archangel': [],
                'MARVEL-Freak': [],
                'MARVEL-Hepzibah': [],
                'MARVEL-Mach_IV': [],
                'MARVEL-Serpent_Society': [],
                'MARVEL-Spectrum': [],
                'MARVEL-The_Hand': [],
                'MARVEL-Vanisher_(Ultimate)': [],
                'VCVA-5.5u1-11': ['00:0c:29:9d:a3:8c'],
                'VCVA-5.5u1-14': ['00:0c:29:75:21:2e'],
                'VCVA33': ['00:0c:29:e3:f9:f7'],
                'VCVA36': ['00:0c:29:44:8b:76'],
                'VCVA37-v5.0': ['00:50:56:b4:89:db'],
                'box': ['00:50:56:82:28:7d'],
                'box_copy': ['00:50:56:82:34:02'],
                'esx4.1.0': ['00:0c:29:1f:ec:ba', '00:0c:29:1f:ec:c4']}

        # see: http://python3porting.com/noconv.html
        si = connect.SmartConnect(host='vcsa',
                                  user='my_user',
                                  pwd='my_password')

        content = si.RetrieveContent()
        virtual_machines = content.viewManager.CreateContainerView(
            content.rootFolder, [vim.VirtualMachine], True)
        for virtual_machine in virtual_machines.view:
            name = virtual_machine.name
            self.assertTrue(name in data.keys())
            macs = data[name]
            if virtual_machine.guest:
                for net in virtual_machine.guest.net:
                    self.assertTrue(net.macAddress in macs)



# -*- coding: utf-8 -*-

# VMware vSphere Python SDK
# Copyright (c) 2008-2015 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import tests
from vcr.stubs import VCRHTTPSConnection
from vcr import config

from pyVmomi import SoapAdapter
from pyVmomi import SoapStubAdapter
from pyVmomi import vim
from pyVmomi.VmomiSupport import GetRequestContext



class SerializerTests(tests.VCRTestBase):
    def test_serialize_object(self):
        val = vim.vm.device.VirtualDeviceSpec.FileOperation()
        # This line should not raise an exception, especially on Python 3.
        SoapAdapter.Serialize(val)

    def test_serialize_integer(self):
        lp = vim.LongPolicy()
        lp.inherited = False
        lp.value = 100
        SoapAdapter.Serialize(lp, version='vim.version.version10')

    def test_serialize_float(self):
        pc = vim.host.VsanInternalSystem.PolicyCost()
        pc.diskSpaceToAddressSpaceRatio = 1.0
        SoapAdapter.Serialize(pc, version='vim.version.version10')

    def test_serialize_unicode(self):
        self.assertEqual(SoapAdapter.SerializeToUnicode('Ḃ'),
                         u'<object xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns="urn:vim25" xsi:type="xsd:string">\u1e02</object>')
        self.assertEqual(SoapAdapter.SerializeToUnicode(u'Ḃ'),
                         u'<object xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns="urn:vim25" xsi:type="xsd:string">\u1e02</object>')
        self.assertEqual(SoapAdapter.SerializeToUnicode(u'\u1e02'),
                         u'<object xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns="urn:vim25" xsi:type="xsd:string">\u1e02</object>')

    def _base_serialize_test(self, soap_creator, request_matcher):
        my_vcr = config.VCR(
            custom_patches=(
                (SoapAdapter, '_HTTPSConnection', VCRHTTPSConnection),))
        my_vcr.register_matcher('request_matcher', request_matcher)

        with my_vcr.use_cassette(
                'test_simple_request_serializer.yaml',
                cassette_library_dir=tests.fixtures_path,
                record_mode='none',
                match_on=['request_matcher']) as cass:
            stub = soap_creator()
            si = vim.ServiceInstance("ServiceInstance", stub)
            content = si.RetrieveContent()
            self.assertTrue(content is not None)
            self.assertTrue(
                '<_this type="ServiceInstance">ServiceInstance</_this>'
                in cass.requests[0].body.decode("utf-8"))

    def _body_request_matcher(self, r1, r2):
        soap_msg = ('<soapenv:Body>'
                    '<RetrieveServiceContent xmlns="urn:vim25">'
                    '<_this type="ServiceInstance">'
                    'ServiceInstance'
                    '</_this>'
                    '</RetrieveServiceContent>'
                    '</soapenv:Body>')
        if soap_msg in r1.body.decode("utf-8"):
            return True
        raise SystemError('serialization error occurred')

    def _request_context_request_matcher(self, r1, r2):
        request_context = ('<soapenv:Header><vcSessionCookie>123456789</vcSessionCookie></soapenv:Header>')
        if request_context in r1.body.decode("utf-8"):
            return True
        raise SystemError('serialization error occurred')

    def test_simple_request_serializer(self):
        def soap_creator():
            return SoapStubAdapter('vcsa', 443)
        self._base_serialize_test(soap_creator, self._body_request_matcher)

    def test_request_context_serializer_instance(self):
        def request_matcher(r1, r2):
            return self._request_context_request_matcher(r1, r2) and self._body_request_matcher(r1, r2)
        def soap_creator():
            return SoapStubAdapter('vcsa', 443, requestContext={'vcSessionCookie': '123456789'})
        self._base_serialize_test(soap_creator, request_matcher)

    def test_request_context_serializer_global(self):
        def request_matcher(r1, r2):
            return self._request_context_request_matcher(r1, r2) and self._body_request_matcher(r1, r2)
        def soap_creator():
            return SoapStubAdapter('vcsa', 443)
        GetRequestContext()['vcSessionCookie'] = '123456789'
        try:
            self._base_serialize_test(soap_creator, request_matcher)
        finally:
            GetRequestContext().pop("vcSessionCookie")




# VMware vSphere Python SDK
# Copyright (c) 2016 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pyVmomi import vim
from pyVmomi.SoapAdapter import SoapStubAdapter, SoapResponseDeserializer
import unittest

#TODO: Fix the test Post 6.7 release
# class DeserializerTests(unittest.TestCase):
#
#     def test_deserialize_unknown_managed_method(self):
#         with open('tests/files/unknown_method.xml', 'rb') as f:
#             data = f.read()
#         stub = SoapStubAdapter(version="vim.version.version6")
#         deserializer = SoapResponseDeserializer(stub)
#         result = vim.TaskHistoryCollector._GetMethodInfo("ReadNext").result
#         obj = deserializer.Deserialize(data, result)
#         print(obj)
#         with self.assertRaisesRegexp(Exception, "Managed method LeaseMapDiskRegion is not available"):
#             obj[-1].name()




#!/usr/bin/env python
# VMware vSphere Python SDK
# Copyright (c) 2008-2015 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Python program for listing the vms on an ESX / vCenter host
"""

from __future__ import print_function

from pyVim.connect import SmartConnect, Disconnect
from pyVmomi import vim

import argparse
import atexit
import getpass
import ssl

def GetArgs():
   """
   Supports the command-line arguments listed below.
   """
   parser = argparse.ArgumentParser(
       description='Process args for retrieving all the Virtual Machines')
   parser.add_argument('-s', '--host', required=True, action='store',
                       help='Remote host to connect to')
   parser.add_argument('-o', '--port', type=int, default=443, action='store',
                       help='Port to connect on')
   parser.add_argument('-u', '--user', required=True, action='store',
                       help='User name to use when connecting to host')
   parser.add_argument('-p', '--password', required=False, action='store',
                       help='Password to use when connecting to host')
   args = parser.parse_args()
   return args


def PrintVmInfo(vm, depth=1):
   """
   Print information for a particular virtual machine or recurse into a folder
   or vApp with depth protection
   """
   maxdepth = 10

   # if this is a group it will have children. if it does, recurse into them
   # and then return
   if hasattr(vm, 'childEntity'):
      if depth > maxdepth:
         return
      vmList = vm.childEntity
      for c in vmList:
         PrintVmInfo(c, depth+1)
      return

   # if this is a vApp, it likely contains child VMs
   # (vApps can nest vApps, but it is hardly a common usecase, so ignore that)
   if isinstance(vm, vim.VirtualApp):
      vmList = vm.vm
      for c in vmList:
         PrintVmInfo(c, depth + 1)
      return

   summary = vm.summary
   print("Name       : ", summary.config.name)
   print("Path       : ", summary.config.vmPathName)
   print("Guest      : ", summary.config.guestFullName)
   annotation = summary.config.annotation
   if annotation != None and annotation != "":
      print("Annotation : ", annotation)
   print("State      : ", summary.runtime.powerState)
   if summary.guest != None:
      ip = summary.guest.ipAddress
      if ip != None and ip != "":
         print("IP         : ", ip)
   if summary.runtime.question != None:
      print("Question  : ", summary.runtime.question.text)
   print("")

def main():
   """
   Simple command-line program for listing the virtual machines on a system.
   """

   args = GetArgs()
   if args.password:
      password = args.password
   else:
      password = getpass.getpass(prompt='Enter password for host %s and '
                                        'user %s: ' % (args.host,args.user))

   context = None
   if hasattr(ssl, '_create_unverified_context'):
      context = ssl._create_unverified_context()
   si = SmartConnect(host=args.host,
                     user=args.user,
                     pwd=password,
                     port=int(args.port),
                     sslContext=context)
   if not si:
       print("Could not connect to the specified host using specified "
             "username and password")
       return -1

   atexit.register(Disconnect, si)

   content = si.RetrieveContent()
   for child in content.rootFolder.childEntity:
      if hasattr(child, 'vmFolder'):
         datacenter = child
         vmFolder = datacenter.vmFolder
         vmList = vmFolder.childEntity
         for vm in vmList:
            PrintVmInfo(vm)
   return 0

# Start program
if __name__ == "__main__":
   main()



# VMware vSphere Python SDK
# Copyright (c) 2008-2015 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This module is a converter from dynamic type to pyVmomi type
"""
__author__ = "VMware, Inc."

from pyVmomi import VmomiSupport, vmodl
from Cache import Cache

## Dynamic type importer
#
class DynamicTypeImporter:
   """ Dynamic type importer  """

   ## Constructor
   #
   # @param  stub Server stub
   def __init__(self, stub, hostSystem=None):
      self.stub = stub
      self.hostSystem = hostSystem

   ## Get dynamic type manager
   #
   # @return moRef to dynamic type manager
   @Cache
   def GetTypeManager(self):
      """ Get dynamic type manager """
      dynTypeMgr = None
      if self.hostSystem:
         try:
            dynTypeMgr = self.hostSystem.RetrieveDynamicTypeManager()
         except vmodl.fault.MethodNotFound as err:
            pass

      if not dynTypeMgr:
         # Older host not support RetrieveDynamicTypeManager
         cmdlineTypesMoId = "ha-dynamic-type-manager"
         dynTypeMgr = vmodl.reflect.DynamicTypeManager(cmdlineTypesMoId,
                                                       self.stub)
      return dynTypeMgr

   ## Import dynamic types
   #
   # @param  prefix Only types with the specified prefix are imported
   # @return dynamic types
   @Cache
   def ImportTypes(self, prefix=''):
      """ Build dynamic types """
      # Use QueryTypeInfo to get all types
      dynTypeMgr = self.GetTypeManager()
      filterSpec = None
      if prefix != '':
         filterSpec = vmodl.reflect.DynamicTypeManager.TypeFilterSpec(
                                                              typeSubstr=prefix)
      allTypes = dynTypeMgr.QueryTypeInfo(filterSpec)

      ## Convert dynamic types to pyVmomi types
      #
      DynamicTypeConstructor().CreateTypes(allTypes)
      return allTypes


## Construct pyVmomi types from dynamic types definition
#
class DynamicTypeConstructor:
   """ Dynamic type constructor  """

   _mapFlags = { "optional": VmomiSupport.F_OPTIONAL,
                 "linkable": VmomiSupport.F_LINKABLE,
                 "link":     VmomiSupport.F_LINK,
                 "secret":   VmomiSupport.F_SECRET }

   ## Constructor
   #
   def __init__(self):
      """ Constructor """
      pass

   ## Create pyVmomi types from vmodl.reflect.DynamicTypeManager.AllTypeInfo
   #
   # @param  allTypes vmodl.reflect.DynamicTypeManager.AllTypeInfo
   def CreateTypes(self, allTypes):
      """
      Create pyVmomi types from vmodl.reflect.DynamicTypeManager.AllTypeInfo
      """
      enumTypes, dataTypes, managedTypes = self._ConvertAllTypes(allTypes)
      self._CreateAllTypes(enumTypes, dataTypes, managedTypes)

   ## Convert all dynamic types to pyVmomi type definitions
   #
   # @param  allTypes vmodl.reflect.DynamicTypeManager.AllTypeInfo
   # @return a tuple of (enumTypes, dataTypes, managedTypes)
   def _ConvertAllTypes(self, allTypes):
      """ Convert all dynamic types to pyVmomi type definitions """
      # Generate lists good for VmomiSupport.CreateXYZType
      enumTypes = self._Filter(self._ConvertEnumType, allTypes.enumTypeInfo)
      dataTypes = self._Filter(self._ConvertDataType, allTypes.dataTypeInfo)
      managedTypes = self._Filter(self._ConvertManagedType,
                                  allTypes.managedTypeInfo)
      retAllTypes = (enumTypes, dataTypes, managedTypes)
      return retAllTypes

   ## Create pyVmomi types from pyVmomi type definitions
   #
   # @param  enumTypes pyVmomi enum type definitions
   # @param  dataTypes pyVmomi data type definitions
   # @param  managedTypes pyVmomi managed type definitions
   def _CreateAllTypes(self, enumTypes, dataTypes, managedTypes):
      """ Create pyVmomi types from pyVmomi type definitions """

      # Create versions
      for typeInfo in managedTypes:
         name = typeInfo[0]
         version = typeInfo[3]
         VmomiSupport.AddVersion(version, '', '1.0', 0, name)
         VmomiSupport.AddVersionParent(version, 'vmodl.version.version0')
         VmomiSupport.AddVersionParent(version, 'vmodl.version.version1')
         VmomiSupport.AddVersionParent(version, version)

      # Create partial types
      for fn, infos in (VmomiSupport.CreateEnumType, enumTypes), \
                       (VmomiSupport.CreateDataType, dataTypes), \
                       (VmomiSupport.CreateManagedType, managedTypes):
         for typeInfo in infos:
            try:
               fn(*typeInfo)
            except Exception as err:
               #Ignore errors due to duplicate importing
               pass

   def _ConvertAnnotations(self, annotations):
      """ Convert annotations to pyVmomi flags """
      flags = 0
      if annotations:
         for annotation in annotations:
            flags |= self._mapFlags.get(annotation.name, 0)
      return flags

   @staticmethod
   def _Filter(fn, types):
      """ Call fn for each non null element in types. Similiar to filter """
      if types:
         return [fn(prop) for prop in types if prop is not None]
      else:
         return []

   def _ConvertParamType(self, paramType):
      """
      Convert vmodl.reflect.DynamicTypeManager.ParamTypeInfo to pyVmomi param
      definition
      """
      if paramType:
         name = paramType.name
         version = paramType.version
         aType = paramType.type
         flags = self._ConvertAnnotations(paramType.annotation)
         privId = paramType.privId
         param = (name, aType, version, flags, privId)
      else:
         param = None
      return param

   def _ConvertMethodType(self, methodType):
      """
      Convert vmodl.reflect.DynamicTypeManager.MethodTypeInfo to pyVmomi method
      definition
      """
      if methodType:
         name = methodType.name
         wsdlName = methodType.wsdlName
         version = methodType.version
         params = self._Filter(self._ConvertParamType, methodType.paramTypeInfo)
         privId = methodType.privId
         faults = methodType.fault

         # Figure out reture info
         if methodType.returnTypeInfo:
            returnTypeInfo = methodType.returnTypeInfo
            retFlags = self._ConvertAnnotations(returnTypeInfo.annotation)
            methodRetType = returnTypeInfo.type
         else:
            retFlags = 0
            methodRetType = "void"
         if wsdlName.endswith("_Task"):
            # TODO: Need a seperate task return type for task, instead of
            #       hardcode vim.Task as return type
            retType = "vim.Task"
         else:
            retType = methodRetType
         retInfo = (retFlags, retType, methodRetType)

         method = (name, wsdlName, version, params, retInfo, privId, faults)
      else:
         method = None
      return method

   def _ConvertManagedPropertyType(self, propType):
      """
      Convert vmodl.reflect.DynamicTypeManager.PropertyTypeInfo to pyVmomi
      managed property definition
      """
      if propType:
         name = propType.name
         version = propType.version
         aType = propType.type
         flags = self._ConvertAnnotations(propType.annotation)
         privId = propType.privId
         prop = (name, aType, version, flags, privId)
      else:
         prop = None
      return prop

   def _ConvertManagedType(self, managedType):
      """
      Convert vmodl.reflect.DynamicTypeManager.ManagedTypeInfo to pyVmomi
      managed type definition
      """
      if managedType:
         vmodlName = managedType.name
         wsdlName = managedType.wsdlName
         version = managedType.version
         parent = managedType.base[0]
         props = self._Filter(self._ConvertManagedPropertyType, managedType.property)
         methods = self._Filter(self._ConvertMethodType, managedType.method)
         moType = (vmodlName, wsdlName, parent, version, props, methods)
      else:
         moType = None
      return moType

   def _ConvertDataPropertyType(self, propType):
      """
      Convert vmodl.reflect.DynamicTypeManager.PropertyTypeInfo to pyVmomi
      data property definition
      """
      if propType:
         name = propType.name
         version = propType.version
         aType = propType.type
         flags = self._ConvertAnnotations(propType.annotation)
         prop = (name, aType, version, flags)
      else:
         prop = None
      return prop

   def _ConvertDataType(self, dataType):
      """
      Convert vmodl.reflect.DynamicTypeManager.DataTypeInfo to pyVmomi data
      type definition
      """
      if dataType:
         vmodlName = dataType.name
         wsdlName = dataType.wsdlName
         version = dataType.version
         parent = dataType.base[0]
         props = self._Filter(self._ConvertDataPropertyType, dataType.property)
         doType = (vmodlName, wsdlName, parent, version, props)
      else:
         doType = None
      return doType

   def _ConvertEnumType(self, enumType):
      """
      Convert vmodl.reflect.DynamicTypeManager.EnumTypeInfo to pyVmomi enum
      type definition
      """
      if enumType:
         vmodlName = enumType.name
         wsdlName = enumType.wsdlName
         version = enumType.version
         values = enumType.value
         enumType = (vmodlName, wsdlName, version, values)
      else:
         enumType = None
      return enumType




# VMware vSphere Python SDK
# Copyright (c) 2017 VMware, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ssl
import unittest
import tests

from pyVmomi import pbm, VmomiSupport, SoapStubAdapter
from pyVim.connect import SmartConnectNoSSL


class PBMTests(tests.VCRTestBase):
    def get_pbm_connection(self, vpxd_stub):
        VmomiSupport.GetRequestContext()["vcSessionCookie"] = \
            vpxd_stub.cookie.split('"')[1]
        hostname = vpxd_stub.host.split(":")[0]
        pbm_stub = SoapStubAdapter(
            host=hostname,
            version="pbm.version.version11",
            path="/pbm/sdk",
            poolSize=0,
            sslContext=ssl._create_unverified_context())
        pbm_si = pbm.ServiceInstance("ServiceInstance", pbm_stub)
        pbm_content = pbm_si.RetrieveContent()
        return pbm_content

    def get_profile(self, profile_name, pbm_content):
        pm = pbm_content.profileManager
        profile_ids = pm.PbmQueryProfile(
            resourceType=pbm.profile.ResourceType(resourceType="STORAGE"),
            profileCategory="REQUIREMENT")
        if len(profile_ids) > 0:
            profiles = pm.PbmRetrieveContent(profileIds=profile_ids)
            for profile in profiles:
                if profile_name in profile.name:
                    return profile
        raise Exception('Profile not found')

    @tests.VCRTestBase.my_vcr.use_cassette('pbm_check_compatibility.yaml',
                                           cassette_library_dir=tests.fixtures_path,
                                           record_mode='Once')
    def test_pbm_check_compatibility(self):

        si = SmartConnectNoSSL(host='vcsa',
                               user='Administrator@vsphere.local',
                               pwd='Admin!23')

        # Connect to SPBM Endpoint
        pbm_content = self.get_pbm_connection(si._stub)

        sp = self.get_profile("test-profile", pbm_content)
        pbm_content.placementSolver.PbmCheckCompatibility(profile=sp.profileId)


if __name__ == '__main__':
    unittest.main()
